import QtQuick 2.3
import QtQuick.Controls 1.2
import QtQuick.Controls.Styles 1.2
import QtQuick.LocalStorage 2.0

Text {
    id: groupselector
    visible: tunearea.currentTune
    signal updated()
    property var contents: []
    function isInGroup(title) {
        var index = contents.indexOf(title);
        return index !== -1;
    }
    onUpdated: {
        if (tunearea.currentTune && isInGroup(tunearea.currentTune.title))
            color = "green";
        else
            color = "black";
    }

    color: {
        if (tunearea.currentTune && isInGroup(tunearea.currentTune.title))
            return "green";
        return "black";
    }

    font.pixelSize: Math.floor(12 * settings.list_scale)
    Rectangle {
        width: parent.width
        height: actionbar.height
        anchors.centerIn: parent
        opacity: settings.buttonOpacity
        color: settings.buttonColor
        radius: settings.buttonRadius
        MouseArea {
            anchors.fill: parent
            //width: parent.width
            //height: actionbar.height
            //anchors.centerIn: parent
            onClicked: {
                if (groupselector.isInGroup(tunearea.currentTune.title))
                    groupselector.removeFromGroup(groupselector.groupName, tunearea.currentTune, tunes.book);
                else
                    groupselector.addToGroup(groupselector.groupName, tunearea.currentTune, tunes.book);
            }
        }
    }
    function addTo(title) {
        contents.push(title);
        updated();
        if (contents.length === 1)
            tunes.tunesItem.addGroup(extendedGroupName);
        if (tunes.selectedRhythm === extendedGroupName)
            tunes.populateTunesList(extendedGroupName);
    }
    function removeFrom(title) {
        var index = contents.indexOf(title);
        contents.splice(index, 1);
        updated();
        if (contents.length === 0)
            tunes.tunesItem.removeGroup(extendedGroupName);
        if (tunes.selectedRhythm === extendedGroupName)
            tunes.populateTunesList(extendedGroupName);
    }
    function addToGroup(group, tune, book) {
        addTo(tune.title);
        insertGroupMember(book, group, tune.title);
    }
    function removeFromGroup(group, tune, book) {
        removeFrom(tune.title);
        eraseGroupMember(book, group, tune.title);
    }
    function insertGroupMember(book, group, title) {
        var databaseName = book + "Groups";
        var databaseTitle = book + " Groups";
        var db = LocalStorage.openDatabaseSync(databaseName, "1.0", databaseTitle, 500 * 20);
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS Groups(groupname TEXT, title TEXT)');
                tx.executeSql('INSERT INTO Groups VALUES(?, ?)', [ group, title ]);
            });
    }
    function eraseGroupMember(book, group, title) {
        var databaseName = book + "Groups";
        var databaseTitle = book + " Groups";
        var db = LocalStorage.openDatabaseSync(databaseName, "1.0", databaseTitle, 500 * 200);
        db.transaction(
            function(tx) {
                tx.executeSql('DELETE FROM Groups WHERE groupname = ? AND title = ?', [ group, title ]);
            });
    }
    function load(book) {
        var databaseName = book + "Groups";
        var databaseTitle = book + " Groups";
        var db = LocalStorage.openDatabaseSync(databaseName, "1.0", databaseTitle, 500 * 200);
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS Groups(groupname TEXT, title TEXT)');
                var rs = tx.executeSql('SELECT * FROM Groups');
                for (var i = 0; i < rs.rows.length; i++)
                {
                    if (rs.rows.item(i).groupname === groupName)
                        addTo(rs.rows.item(i).title);
                    //console.log(rs.rows.item(i).groupname + ":" + rs.rows.item(i).title);
                }
            });
    }

}
