import QtQuick 2.3
import QtQuick.Controls 1.2
import QtQuick.Layouts 1.1

ColumnLayout {
    //property alias filelist: filelist
    visible: false
    property bool fileMode: false
    property int calculatedWidth: settings.getTextExtent(parent, Math.floor(12 * settings.list_scale), "No ABC files found.  If you know the file path, enter it into the file name field below.").width;
    spacing: 30
    property alias bookfieldtext: bookfield.text
    //Layout.preferredWidth: getTextExtent(parent, pixelSize, "http://www.pghardy.net/concertina/tunebooks/pgh_session_tunebook.abc");
    function clear() {
        fileMode = false;
        urlfield.text = "";
        titlefield.text = "";
        bookfield.text = "";
    }
    function setFields(url, title, book) {
        urlfield.text = url;
        if (urlfield.text === "")
            state = "editing file";
        else
            state = "editing url";
        titlefield.text = title;
        bookfield.text = book;
    }
    states: [
        State {
            name: "editing file"
        },
        State {
            name: "editing url"
        }
    ]

   TextField {
        visible: !bookentryform.fileMode && parent.state !== "editing file"
        id: urlfield
        Layout.preferredWidth: parent.calculatedWidth  //disclaimer.width
        placeholderText: "Download Url (http://...)"
        //text: "http://www.pghardy.net/concertina/tunebooks/williamclarke_tunes.abc"
        onTextChanged: {
            if (library.isValidUrl(text)) {
                bookfield.createText();
                addbooknow.enable();
            }
            else {
                bookfield.text = "Url must end with '.abc'";
                addbooknow.disable();
            }
        }
        MouseArea {
            anchors.fill: parent
            propagateComposedEvents: true
            onPressAndHold: {
                if (urlfield.canPaste)
                    urlfield.paste();
            }
        }
    }
    TextField {
        id: titlefield
        Layout.preferredWidth: parent.calculatedWidth  //disclaimer.width
        placeholderText: "Title (eg John's Collection)"
        //text: "William Clarke of Feltwell Tunebook"
        function createText() {
            text = library.createTitleFromPath(bookfield.text);
        }
    }
    TextField {
        id: bookfield
        Layout.preferredWidth: parent.calculatedWidth  //disclaimer.width
        placeholderText: "Book (file name)"
        onTextChanged: {
            if (bookentryform.fileMode || parent.state === "editing file")
            {
                if (library.isValidFile(text)) {
                    titlefield.createText();
                    addbooknow.enable();
                }
                else {
                    titlefield.text = "File name must end with '.abc'";
                    addbooknow.disable();
                }
            }
            else
                library.createPathFromUrl(urlfield.text);
        }
        function createText() {
            var t = urlfield.text;
            text = t.replace(/^.*[\\\/]/, '');
            titlefield.createText();
        }
    }
    RowLayout {
        spacing: 50
        Layout.alignment: Qt.AlignHCenter
        Text {
            id: addbooknow
            Layout.alignment: Qt.AlignLeft
            //visible: bookentryform.titlefield.text
            text: booklistmodel.editingItem === -1 ? "Add book now" : "Update book now"
            font.pixelSize: Math.floor(16 * settings.list_scale)
            color: "gray"
            function enable() {
                addbooknow.color = "black";
                addbooknowmouse.enabled = true;
            }
            function disable() {
                addbooknow.color = "gray";
                addbooknowmouse.enabled = false;
            }

            MouseArea {
                id: addbooknowmouse
                enabled: false
                anchors.fill: parent
                onClicked: {
                    if (booklistmodel.editingItem === -1) {
                        booklistmodel.addTunebook(titlefield.text, bookfield.text, urlfield.text);
                    }
                    else {
                        booklistmodel.updateTunebook(booklistmodel.editingItem, titlefield.text, bookfield.text, urlfield.text);
                    }
                    bookentryform.visible = false;
                    bookentryform.fileMode = false;
                }
            }
        }
//                Rectangle {
//                    Layout.fillWidth: true
//                }
        Text {
            Layout.alignment: Qt.AlignRight
            //visible: bookentryform.titlefield.text
            text: "Cancel"
            font.pixelSize: Math.floor(16 * settings.list_scale)
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (booklistmodel.editingItem === -1)
                        bookentryform.visible = false;
                    else
                        booklistmodel.cancelEditing();
                    bookentryform.fileMode = false;
                }
            }
        }
    }
}
