import QtQuick 2.3
import QtQuick.Controls 1.2
import QtQuick.Layouts 1.1


ListView {
    property int titlePixelSize: Math.floor(30 * settings.list_scale)
    property real actionPixelSize: Math.floor(15 * settings.list_scale)
    property var titleWidth: calcTitleWidth(booklistmodel, liblist, titlePixelSize)
    property var link1Width: calcImageAndTextLinkWidth(parent, actionPixelSize, "open")
    property var link2Width: calcImageAndTextLinkWidth(parent, actionPixelSize, "download")
    property var link3Width: settings.sharingEnabled ? calcImageAndTextLinkWidth(parent, actionPixelSize, "share") : 0
    //height: mainview.height
    //Layout.preferredHeight: mainview.height / 2
    Layout.preferredWidth: titleWidth + link1Width + link2Width + link3Width
    //width: titleWidth + link1Width + link2Width
    model: booklistmodel
    property int pixelSizeActionText: Math.floor(12 * settings.list_scale)
    property real actionImageScale: 0.5 * settings.list_scale
    function calcTitleWidth(model, parent, pixelSize) {
        //console.log("calcTitleWidth")
        var i;
        var maxWidth = 0;
        for (i = 0; i < model.count; i++) {
            var data = model.get(i);
            var extent = settings.getTextExtent(parent, pixelSize, data.title)
            if (extent.width > maxWidth)
                maxWidth = extent.width;
        }
        return maxWidth;
    }
    function calcImageAndTextLinkWidth(parent, pixelSize, text) {
        //console.log("calcImageAndTextLinkWidth")
        var extent = settings.getTextExtent(parent, pixelSize, text);
        //width: height * 1.5 + text.implicitWidth
        return extent.height * 1.5 + extent.width;
    }
    delegate: liblistdelegate
    Component {
        id: liblistdelegate
        Row {
            Button {
                id: deletebutton
                anchors.verticalCenter: parent.verticalCenter
                visible: library.editMode && !model.permanent
                text: "Delete"
                onClicked: {
                    booklistmodel.deleteItem(index);
                }
            }
            Button {
                id: editbutton
                anchors.verticalCenter: parent.verticalCenter
                visible: library.editMode && !model.permanent
                text: "Edit"
                onClicked: {
                    booklistmodel.editItem(index);
                }
            }

            ColumnLayout {
                spacing: 1
                anchors.verticalCenter: parent.verticalCenter
                width: liblist.titleWidth   //300 * settings.list_scale
                Text {
                    id: text
                    //anchors.verticalCenter: parent.verticalCenter
                    width: liblist.titleWidth   //300 * settings.list_scale
                    text: title
                    font.pixelSize: liblist.titlePixelSize
                }
                Text {
                    id: urltext
                    visible: library.editMode
                    //anchors.verticalCenter: parent.verticalCenter
                    width: liblist.titleWidth   //300 * settings.list_scale
                    text: url === "" ? book : url
                    font.pixelSize: liblist.titlePixelSize / 3
                    Connections {
                        target: library
                        onEditModeChanged: {
                            urltext.visible = library.editMode
                        }
                    }
                }
            }
            ImageAndTextLink {
                id: link1
                visible: model.downloaded
                //border.width: 1   // for debugging, shows where rectangle is
                color: mainview.color
                pixelSize: liblist.actionPixelSize
                text: "open"
                source: "openbook.png"
                Connections {
                    target: library
                    onClearAllSelected: link1.state = ""
                    onDownloadedChanged: {
                        var downloaded = model.downloaded;
                        link1.visible = downloaded;
                    }
                }
                function open() {
                    if (bookloader.item)
                        bookloader.item.visible = true
                    library.message = "Loading " + model.book + ".  Please wait.";
                    bookloader.bookname = model.book;
                    bookloader.setSource();
                    //bookloader.source = "TuneBook.qml"
                    //parent.state = "selected"
                }
                action: open
            }
            ImageAndTextLink {
                id: link2
                visible: model.url !== ""
                //border.width: 1   // for debugging, shows where rectangle is
                color: mainview.color
                pixelSize: liblist.actionPixelSize
                book: model.book
                url: model.url
                text: library.isBookDownloaded(model.book) ? "update" : "download"
                source: library.isBookDownloaded(model.book) ? "updatebook.png" : "downloadbook.png"
                Connections {
                    target: library
                    onClearAllSelected: link2.state = ""
                    onDownloadedChanged: {
                        var downloaded = model.downloaded;
                        link2.text = downloaded ? "update" : "download"
                        link2.source = downloaded ? "updatebook.png" : "downloadbook.png"
                    }
                }
                function download() {
                    library.downloadBook(model.book, model.url)
                }
                action: download
            }
            ImageAndTextLink {
                id: share
                visible: model.url !== "" && !model.permanent && settings.sharingEnabled
                //border.width: 1   // for debugging, shows where rectangle is
                color: mainview.color
                pixelSize: liblist.actionPixelSize
                book: model.book
                url: model.url
                text: "share"
                source: "share.png"
                Connections {
                    target: library
                    onClearAllSelected: share.state = ""
                }
                function shareurl() {
                    library.shareUrl(model.url, model.title);
                }
                action: shareurl
            }
        }
    }
}
