import QtQuick 2.3
import QtQuick.Layouts 1.1
import Tune 1.0
import Bar 1.0
import Note 1.0
import "scoredraw.js" as Scoredraw
import "notefinder.js" as Notefinder

ColumnLayout {
    id: tuneview
    property Tune currentTune: null
    property int pixelSizeTitleRow: 14
    property alias mergetext: mergemessage.text
    spacing: 5
    function redrawScore(tune) {
        // adjust the scale to fit the tune
        settings.score_scale = 1;       // calcScoreWidth uses settings.score_scale so it needs to be 1
        var width = Scoredraw.calcScoreWidth(tune);     // do not use score.iosScale here
        var height = Scoredraw.calcScoreHeight(tune);   // do not use score.iosScale here
        // we want width to equal flick.width
        // settings.score_scale * width == flick.width
        var hScale = flick.width / width;
        var vScale = flick.height / height;
        // choose the smallest so that we see everything
        settings.score_scale = (hScale < vScale) ? hScale : vScale;
        //console.log("scale: ", settings.score_scale)
        if (tuneview.currentTune !== tune)
            fretboard.setCapoPosition(0);
        tuneview.currentTune = tune;
        score.markDirty(score.canvasWindow);
    }
    function stopPlaying() {
        // if music is playing, stop it
        noteTimer.stop();
    }
    LoadingInfo {
    }

    RowLayout {
        visible: mergemessage.text != ""
        Text {
            id: mergemessage
            //text: "test merge message"
        }
    }
    states: [
        State {
            name: "infohidden"
            PropertyChanges { target: fretboard; visible: false }
            PropertyChanges { target: infoblock; visible: false }
            PropertyChanges { target: meterblock; visible: false }

        }
    ]
    RowLayout {
        id: infoblock
        // title and rhythm
        spacing: 10
        /*Flickable {
            //visible: tuneview.currentTune !== null
            id: titleflick
            anchors.fill: parent
            flickableDirection: Flickable.HorizontalFlick
            Rectangle {
                id: flickrect
                color: "red"
                anchors.fill: parent
                //opacity: 0.3
                opacity: 0
            }

            onMovementStarted: {
                flickrect.opacity = 0.3;
            }
            onMovingChanged: {
                //flickrect.opacity = 1;
            }

            onFlickStarted: {
                //var x = contentX;
                //var y = titleflick.width;
                flickrect.opacity = 0.3;
                var extent = titleflick.width / 25;
                if (contentX > extent)
                    tunes.setPreviousTune(tuneview);
                else if (contentX < -extent)
                    tunes.setNextTune(tuneview);
            }
            onFlickEnded: {
                flickrect.opacity = 0;

            }
        }*/

        Text {
            id: titletext
            text: tuneview.currentTune === null ? "" : tuneview.currentTune.title
            font.pixelSize: Math.floor(16 * settings.text_scale)
            //font.pixelSize: pixelSizeTitleRow
            font.bold: true
        }
        Text {
            text: tuneview.currentTune === null ? "" : tuneview.currentTune.rhythm
            font.pixelSize: Math.floor(10 * settings.text_scale)
            //font.pixelSize: pixelSizeTitleRow
            font.italic: true
        }
        Rectangle {
            opacity: 0
            Layout.fillWidth: true
        }
        ColumnLayout {
            spacing: 1
            Text {
                text: tuneview.currentTune === null ? "" : tuneview.currentTune.composer
                font.pixelSize: Math.floor(6 * settings.text_scale)
                font.italic: true
            }
            Text {
                text: tuneview.currentTune === null ? "" : tuneview.currentTune.origin
                font.pixelSize: Math.floor(6 * settings.text_scale)
                font.italic: true
            }
        }
        Rectangle {
            opacity: 0
            width: 10 * settings.text_scale
        }
    }
    Row {
        // keys, meter, temp and parts
        id: meterblock
        spacing: 20
        property int textsize: Math.floor(10 * settings.text_scale)
        Text {
            text: {
                if (tuneview.currentTune === null)
                    return "";
                else if (tuneview.currentTune.key !== tuneview.currentTune.adjustedKey)
                    return "Original Key: " + tuneview.currentTune.keys;
                else
                    return "Key: " + tuneview.currentTune.keys;
            }
            font.pixelSize: parent.textsize
        }
        /*
        Text {
            text: tuneview.currentTune === null ? "" : "Inc"
            font.pixelSize: Math.floor(12 * settings.text_scale)
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    tuneview.currentTune.incrementKey();
                    score.markDirty(score.canvasWindow);
                }
            }
        }
        Text {
            text: tuneview.currentTune === null ? "" : "Dec"
            font.pixelSize: Math.floor(12 * settings.text_scale)
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    tuneview.currentTune.decrementKey();
                    score.markDirty(score.canvasWindow);
                }
            }
        }
        */
        Text {
            text: tuneview.currentTune === null ? "" : "Meter: " + tuneview.currentTune.meter
            font.pixelSize: parent.textsize
        }
        Text {
            text: {
                if (tuneview.currentTune === null)
                    return ""
                else if (tuneview.currentTune.tempoInitial !== tuneview.currentTune.tempo)
                    "Original Tempo: " + tuneview.currentTune.tempoInitial
                else
                    "Tempo: " + tuneview.currentTune.tempoInitial
            }
            font.pixelSize: parent.textsize
        }
        Text {
            text: tuneview.currentTune === null || tuneview.currentTune.part === "" ? "" : "Parts: " + tuneview.currentTune.part
            font.pixelSize: parent.textsize
        }
    }
    /*
    Row {
        Rectangle {
            visible: tuneview.currentTune !== null;
            height: 50
            width: 150
            color: "light gray"
            Text {
                id: beamtext
                text: "No beams";
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (score.useBeams) {
                        score.useBeams = false;
                        beamtext.text = "Use Beams";
                    }
                    else {
                        score.useBeams = true;
                        beamtext.text = "No Beams";
                    }
                    score.markDirty(score.canvasWindow);

                }
            }
        }
    }

    Playdata
    {
        // bar number etc
    }

    Playcontrols
    {
        // start, stop etc
    }
    */

    Timer {
        id: noteTimer
        repeat: true
        property int count: 0
        property int repeatCountdown: 0
        property int countSticky: 0
        property int barNumber: 0
        property Bar currentBar: null
        property Note currentNote: null
        property int elapsed: 0
        property int overflow: 0
        property int systemCountStart: 0
        property int systemCountEnd: 0
        property int systemCountDuration: 0
        property int countTotal: 0
        property int systemCountPeriod: 0
        property int systemCountPeriodMax: 0
        property int systemCountPeriodMin: 10000
        property bool paused: false
        //property bool continuousPlay: false

        //function setPlayContinuously() {
        //    continuousPlay = continuousPlay ? false : true;
        //}
        function resetPlay() {
            countTotal = 0;
            systemCountPeriod = 0;
            systemCountPeriodMin = 10000;
            systemCountPeriodMax = 0;
            systemCountEnd = 0;
            systemCountDuration = 0;
        }
        function resumePlay() {
//            countTotal = 0;
//            systemCountPeriod = 0;
//            systemCountPeriodMin = 10000;
//            systemCountPeriodMax = 0;
//            systemCountEnd = 0;
//            systemCountDuration = 0;
            systemCountStart = parent.currentTune.systemCount();
        }

        onTriggered: {
            var newOpacity = 0;
            countTotal++;
            systemCountPeriod = (parent.currentTune.systemCount() - systemCountStart)/countTotal;
            if (countTotal > 10) {
                if (systemCountPeriodMax < systemCountPeriod)
                    systemCountPeriodMax = systemCountPeriod;
                if (systemCountPeriodMin > systemCountPeriod)
                    systemCountPeriodMin = systemCountPeriod;
                if (countTotal > 50) {
                    if ((systemCountPeriod - interval) > 0) {
                        newOpacity = (systemCountPeriod - interval) / 10;
                        if (newOpacity > 0.3)
                            newOpacity = 0.3;   // no greater than 0.3
                    }
                    else {
                        newOpacity = 0.0;
                    }
                }
                if (newOpacity !== slowSpeedIndicator.opacity)
                    slowSpeedIndicator.opacity = newOpacity;
            }
            count--;
            if (count === 0) {
                var tied = currentNote.tied;
                if (!tied)
                    currentNote.stopPlay();
                var result = Notefinder.fetchNextNote(currentNote, currentBar, parent.currentTune);
                currentBar = result.bar;
                currentNote = result.note;
                barNumber = result.barNumber;
                if (currentNote === null) {
                    parent.currentTune.stopChords();
                    running = false;
                    systemCountEnd = parent.currentTune.systemCount();
                    systemCountDuration = systemCountEnd - systemCountStart;
                    repeatCountdown--;
                    if (repeatCountdown > 0)
                        tunePlayer.play();
                    else if (parent.currentTune.continuousPlay) {
                        tunes.setNextTune(tuneview);
                        tunePlayer.play();
                    }

                }
                else {
                    // count make have been decremented while we have been here so += instead of =
                    overflow = count;
                    var thisCount = currentNote.startPlay(tied);
                    count += thisCount;
                    countSticky = thisCount;    //count;
                    elapsed += thisCount * interval
                }
                score.highlightNote(currentNote);
            }
            //else
            //    count--;
        }
        function previousBar() {
            var result = Notefinder.fetchFirstNoteInPreviousBar(currentBar, parent.currentTune);
            currentBar = result.bar;
            currentNote = result.note;
            barNumber = result.barNumber;
            score.highlightNote(currentNote);
        }
        function nextBar() {
            var result = Notefinder.fetchFirstNoteInNextBar(currentBar, parent.currentTune);
            currentBar = result.bar;
            currentNote = result.note;
            barNumber = result.barNumber;
            score.highlightNote(currentNote);
        }

        function start() {
            paused = false;
            if (repeatCountdown == 0)
                repeatCountdown = parent.currentTune.repeatCount;
            parent.currentTune.startAudioEngine();
            resetPlay();
            interval = parent.currentTune.timerInterval();
            var result = Notefinder.fetchNextNote(null, null, parent.currentTune);
            currentBar = result.bar;
            currentNote = result.note;
            barNumber = result.barNumber;
            //currentBar = parent.currentTune.firstBar();
            //currentNote = noteTimer.currentBar.firstNote();
            running = true;
            systemCountStart = parent.currentTune.systemCount();
            count = currentNote.startPlay(false);
            countSticky = count;
            elapsed = count * interval;
            score.highlightNote(currentNote);
        }
        function stop() {
            paused = false;
            running = false;
            if (currentNote !== null)
            {
                repeatCountdown = 0;
                currentNote.stopPlay();
                currentNote = null;
                systemCountEnd = parent.currentTune.systemCount();
                systemCountDuration = systemCountEnd - systemCountStart
                score.highlightNote(currentNote);
            }
            if (parent.currentTune)
            {
                parent.currentTune.stopChords();
                parent.currentTune.stopAudioEngine();
            }
        }
        function pause() {
            running = false;
            if (currentNote !== null)
            {
                currentNote.stopPlay();
                paused = true;
            }
        }
        function resume() {
            resumePlay();
            paused = false;
            running = true;
            count = currentNote.startPlay(false);
        }
    }
    Item {
        id: tunePlayer
        function previousBar() {
            if (noteTimer.paused)
                noteTimer.previousBar();
        }
        function nextBar() {
            if (noteTimer.paused)
                noteTimer.nextBar();
        }
        function resume() {
            if (noteTimer.paused)
                noteTimer.resume();
        }
        function pause() {
            if (noteTimer.running)
                noteTimer.pause();
//            else
//                noteTimer.resume();
        }

        function play() {
            if (noteTimer.running || noteTimer.paused)
                noteTimer.stop();
            else
                noteTimer.start();
            //parent.currentTune.play();
        }
    }
    /*
    Row {
        visible: parent.currentTune !== null
        spacing: 10
        Text {
            text: "Previous tune"
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    tunes.setPreviousTune(tuneview);
                }
            }
        }
        Text {
            text: "Next tune"
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    tunes.setNextTune(tuneview);
                }
            }
        }
    }
    Fretboard
    {
        id: fretboard

    }
    */
    FretsAndPlay {
        id: fretboard
    }

    AbcView {
        id: abc
        Layout.alignment: Qt.AlignTop
        Layout.fillWidth: true
        tune: tuneview.currentTune

    }

    /*ListView {
        width: 200
        height: 150
        id: notes
        Layout.alignment: Qt.AlignTop
        model: noteListModel;
        delegate :
            Text {
                width: 200
                text: content
                }
    }*/

    Flickable {
        Layout.fillWidth: true
        Layout.fillHeight: true
        id: flick
        clip: true
        contentHeight: score.height
        contentWidth: score.width
        onWidthChanged:  {
            if (!tunelistAnimation.running) // && !rhythmlistAnimation.running)
                tunes.windowAreaChanged();
        }
        onHeightChanged:  {
            tunes.windowAreaChanged();
        }
        onFlickStarted: {
            //console.log("h: " + horizontalVelocity);
            if (Math.abs(horizontalVelocity) > 700) {
                var extent = flick.width / 8;
                if (contentX < -extent) {
                    tuneview.stopPlaying();
                    tunes.setPreviousTune(tunearea);
                    contentX = 0;
                    contentY = 0;
                }
                else if (contentX > extent) {
                    tuneview.stopPlaying();
                    tunes.setNextTune(tunearea);
                    contentX = 0;
                    contentY = 0;
                }
            }
        //    score.ourTune.pinchStarted();
            /*if (flickingHorizontally) {
                if (Math.abs(horizontalVelocity) > Math.abs(verticalVelocity)) {
                    if (horizontalVelocity < 0)
                        tunes.setNextTune(tunearea);
                    else
                        tunes.setPreviousTune(tunearea);
                }
            }*/
        }
        Canvas {
            //anchors.fill: parent
            property Tune ourTune: tuneview.currentTune
            property real iosScale: Qt.platform.os === "ios" ? 0.5 : 1.0

            width: Scoredraw.calcScoreWidth(ourTune) / iosScale
            height: Scoredraw.calcScoreHeight(ourTune) / iosScale
            //height: 1500 * mainview.scale
            id: score
            property int leftMargin: 20
            property int rightMargin: 20
            property real topMargin: 30.5
            property int verticalPadding: 40
            property int verticalPaddingTab: 20     // no space required for chords
            property int double_bar_spacing: 6 * settings.score_scale
            property int note_spacing: 20 * settings.score_scale
            property int spacing: 10 * settings.score_scale
            //property Note highlitNote: null;
            property string fontChord: (Math.floor(12 * settings.score_scale)).toString() + "px bold sans-serif";
            property string fontTuplet: (Math.floor(10 * settings.score_scale)).toString() + "px italic sans-serif";
            property string fontRepeatEnding: (Math.floor(8 * settings.score_scale)).toString() + "px sans-serif";
            property string fontPart: (Math.floor(12 * settings.score_scale)).toString() + "px serif";
            property bool useBeams: true;
            property string baseValueGuitar: "E"
            property var tuningVectorGuitar: [ 5, 5, 5, 4, 5 ];
            property string baseValueMandolin: "G"
            property var tuningVectorMandolin: [ 7, 7, 7 ];
            property string baseValue: "";
            property var tuningVector: [];
            property bool smartTabbing: false
            property int defaultLegLength: 0
            property int legLength: 15 * settings.score_scale
            property int legXOffset: 4 * settings.score_scale
            property int legYOffset: score.spacing / 4

            //property var tuning: tuningGuitar;
            property bool hideStaff: false

            transform: Scale {
                xScale: score.iosScale
                yScale: score.iosScale
            }

            function smartTabbingIsSet() {
                return smartTabbing;
            }
            function enableSmartTabbing() {
                smartTabbing = true;
                tuneview.redrawScore(ourTune);
            }
            function disableSmartTabbing() {
                smartTabbing = false;
                tuneview.redrawScore(ourTune);
            }

            function guitarTabIsSet() {
                return tuningVector.length === 5;
            }
            function mandolinTabIsSet() {
                return tuningVector.length === 3;
            }
            function showLegsIsSet() {
                if (ourTune === null)
                    return false;
                return ourTune.showLegs;
            }
            function hideStaffViewOnTabIsSet() {
                if (ourTune === null)
                    return false;
                return ourTune.hideStaffWhenTab;
            }

            function setNoTab() {
                baseValue = "";
                tuningVector = [];
                tuneview.redrawScore(ourTune);  // need to rescale
            }
            function setGuitarTab() {
                baseValue = baseValueGuitar;
                tuningVector = tuningVectorGuitar;
                tuneview.redrawScore(ourTune);  // need to rescale
            }
            function setMandolinTab() {
                baseValue = baseValueMandolin;
                tuningVector = tuningVectorMandolin;
                tuneview.redrawScore(ourTune);  // need to rescale
            }

            MouseArea {
                anchors.fill: parent
                onWheel: {
                    if (wheel.angleDelta.y > 0) {
                        settings.score_scale = settings.score_scale * 1.05;
                    }
                    else {
                        settings.score_scale = settings.score_scale / 1.05;
                    }
                }
            }

            PinchArea {
                id: pincharea
                anchors.fill: parent
                pinch.target: parent
                pinch.minimumScale: 0.2
                pinch.maximumScale: 2
                onPinchFinished: {
                    // make the scale permanent
                    settings.score_scale = settings.score_scale * pincharea.pinch.target.scale;
                    pincharea.pinch.target.scale = 1;
                }
            }
            Rectangle {
                id: noteHighlighter
                property bool useMe: true
                visible: false
                color: "black"
                opacity: 0.5
                width: 2
                height: 10
            }
            Rectangle {
                id: slowSpeedIndicator
                anchors.fill: parent
                color: "red"
                opacity: 0.0
                visible: noteTimer.running
                //Behavior on opacity { PropertyAnimation {} }
            }

            function highlightNote(note) {
                if (note) {
                    if (noteHighlighter.useMe) {
                        noteHighlighter.x = note.xPos;
                        noteHighlighter.y = note.top;
                        noteHighlighter.height = note.bottom - note.top;
                        noteHighlighter.visible = true;
                    }
                    else {
                        if (!note.isRest)
                            particleSystem.explode(note.xPos, note.yPos);
                    }
                }
                else {
                    if (noteHighlighter.useMe) {
                        noteHighlighter.visible = false;
                    }
                }
                //highlitNote = note;
                //markDirty(canvasWindow);
            }
            ParticleSystem {
                id: particleSystem
                anchors.fill: parent
                running: false  //noteTimer.running
            }
            onPaint: {
                if (ourTune != null)
                {
                    var i,
                        ctx = score.getContext('2d');
                    if (ourTune.showLegs) {
                        defaultLegLength = legLength;
                    }
                    else {
                        defaultLegLength = 0;
                    }
                    hideStaff = false;
                    //setGuitarTab();
                    ctx.save();
                    var openStringValues = [];
                    var openStringNames = [];
                    if (baseValue !== "")
                    {
                        var openStringValue = ourTune.baseValueToNoteValue(baseValue);
                        openStringValue += fretboard.capoPosition;
                        openStringValues.push(openStringValue);
                        openStringNames.push(Scoredraw.getStringName(baseValue, fretboard.capoPosition));
                        var offset = 0;
                        offset += fretboard.capoPosition;
                        for (i = 0; i < tuningVector.length; i++)
                        {
                            offset += tuningVector[i];
                            openStringValue += tuningVector[i];
                            openStringValues.push(openStringValue);
                            openStringNames.push(Scoredraw.getStringName(baseValue, offset));
                        }
                        if (ourTune.hideStaffWhenTab)
                            hideStaff = true;
                    }
                    ctx.clearRect(0, 0, score.width, score.height);
                    Scoredraw.drawTune(ctx, ourTune, useBeams, openStringValues, openStringNames, smartTabbing);
                    ctx.restore();
                }
            }
        }
    }
}
