import QtQuick 2.3
import QtQuick.Controls 1.2
import QtQuick.Controls.Styles 1.2
import QtQuick.Layouts 1.1
import QtQuick.LocalStorage 2.0
import Tunes 1.0
//import TuneListModel 1.0
import RhythmModel 1.0
import AbcModel 1.0
import Tune 1.0
//import Bar 1.0
import AbcLine 1.0
import Note 1.0
import Tuplet 1.0
import InstrumentListModel 1.0
import TempoListModel 1.0
//import Settings 1.0
//import QtQuick.Window 2.1
import Library 1.0
import "Library"

Rectangle {
    // asus pad infinity tablet: 1920 x 1200
    // samsung 400B laptop: 1366 x 768
    // HTC One X 1280 x 720
    // Nexus 7 (2012) 1280 x 800
    // Nexus 7 (2013) 1900 x 1200
    //property int scale: 2     // for asus infinity
    //property real score_scale: 1.2
    property int scale: 1
    width: 1000
    height: 550 * scale
    id: mainview
    color: "light blue"
    Settings {
        id: settings
    }
    focus: true
    Keys.onPressed: {
        if (bookloader.item)
            bookloader.item.keyPressed(event);
    }
    // we have to handle both width and height change, just handling width change does not work
    onWidthChanged: {
        if (bookloader.item) {
            //console.log("width changed: " + mainview.width + "," + mainview.height)
            bookloader.item.width = mainview.width
            //bookloader.item.height = mainview.height
        }
    }
    onHeightChanged: {
        if (bookloader.item) {
            //console.log("height changed: " + mainview.width + "," + mainview.height)
            //bookloader.item.width = mainview.width
            bookloader.item.height = mainview.height
        }
    }


    states: [
        State {
            name: "hidden"
            PropertyChanges { target: booklistcolumn; visible: false }
            PropertyChanges { target: mainview; color: "white" }
            //PropertyChanges { target: library; message: "" }
//            PropertyChanges { target: tunebook; width: mainview.width }
//            PropertyChanges { target: tunebook; height: mainview.height }
        },
        State {
            name: "visible"
            PropertyChanges { target: mainview; color: "light blue" }
            PropertyChanges { target: bookloader; source: "" }
            PropertyChanges { target: booklistcolumn; visible: true }
        }
    ]

    Loader {
        id: bookloader
        property string bookname        // with .abc
        property string shortbookname   // without .abc
        onBooknameChanged: {
            // remove the .abc
            shortbookname = bookname.slice(0, -4);
        }
        function setSource() {
            source = "TuneBook.qml";
        }

        asynchronous: true
        onLoaded: {
            library.clearAllSelected();
            library.message = "";
            mainview.state = "hidden"
            item.width = mainview.width
            item.height = mainview.height
        }
    }
    ListModel {
        id: booklistmodel
        property int editingItem: -1
        property bool hasEditableItems: false
        function editItem(index) {
            var data = get(index);
            bookentryform.setFields(data.url, data.title, data.book);
//            urlfield.text = data.url;
//            titlefield.text = data.title;
//            bookfield.text = data.book;
            bookentryform.visible = true
            editingItem = index;
        }
        function cancelEditing() {
            bookentryform.visible = false
            editingItem = -1;
        }

        function deleteItem(index) {
            var data = get(index);
            confirmdeletion.show(index, data.title, data.book, data.url);
        }
        function deleteItemConfirmed(index) {
            var data = get(index);
            eraseTunebook(data.title, data.book, data.url);
            remove(index, 1);
        }

        function updateTunebook(index, title, book, url) {
            var data = get(index);
            var db = LocalStorage.openDatabaseSync("AbcSessionViewerBooks", "1.0", "Abc Session Viewer Books", 200);
            db.transaction(
                function(tx) {
                    var whereClause = 'title = "' + data.title + '" AND book = "' + data.book + '" AND url = "' + data.url + '"';
                    var setClause = 'title = "' + title + '", book = "' + book + '", url = "' + url + '"';
                    var command = 'UPDATE Books SET ' + setClause + ' WHERE ' + whereClause;
                    //console.log(command);
                    tx.executeSql(command);
                });
            set(index, {"title": title, "book": book, "url": url});
            library.updateDownloaded(booklistmodel);
        }
        function addTunebook(title, book, url) {
            booklistmodel.append({ "title" :  title, "book" : book, "url" : url, "downloaded" : false, "permanent" : false });
            insertTunebook(title, book, url);
            library.updateDownloaded(booklistmodel);
            liblist.currentIndex = booklistmodel.count - 1;
        }

        function eraseTunebook(title, book, url) {
            var db = LocalStorage.openDatabaseSync("AbcSessionViewerBooks", "1.0", "Abc Session Viewer Books", 200);
            db.transaction(
                function(tx) {
                    var command = 'DELETE FROM Books WHERE title = "' + title + '" AND book = "' + book + '" AND url = "' + url + '"';
                    tx.executeSql(command);
                });
        }
        function insertTunebook(title, book, url) {
            var db = LocalStorage.openDatabaseSync("AbcSessionViewerBooks", "1.0", "Abc Session Viewer Books", 200);
            db.transaction(
                function(tx) {
                    tx.executeSql('CREATE TABLE IF NOT EXISTS Books(title TEXT, book TEXT, url Text)');
                    tx.executeSql('INSERT INTO Books VALUES(?, ?, ?)', [ title, book, url ]);
                    booklistmodel.hasEditableItems = true;
                });
        }
        function loadTunebooks() {
            var db = LocalStorage.openDatabaseSync("AbcSessionViewerBooks", "1.0", "Abc Session Viewer Books", 200);
            db.transaction(
                function(tx) {
                    //tx.executeSql('DROP TABLE Books');
                    tx.executeSql('CREATE TABLE IF NOT EXISTS Books(title TEXT, book TEXT, url Text)');
                    var rs = tx.executeSql('SELECT * FROM Books');
                    for (var i = 0; i < rs.rows.length; i++)
                    {
                        var title = rs.rows.item(i).title;
                        var book = rs.rows.item(i).book;
                        var url = rs.rows.item(i).url;
                        booklistmodel.append({ "title" :  title, "book" : book, "url" : url, "permanent" : false });
                        booklistmodel.hasEditableItems = true;
                    }
                });
        }
        ListElement {
            title: "Paul Hardy's Session Tunebook"
            book: "pgh_session_tunebook.abc"
            url: "http://www.pghardy.net/concertina/tunebooks/pgh_session_tunebook.abc"
            downloaded: false
            permanent: true
        }
        ListElement {
            title: "Paul Hardy's Annex Tunebook"
            book: "pgh_annex_tunebook.abc"
            url: "http://www.pghardy.net/concertina/tunebooks/pgh_annex_tunebook.abc"
            downloaded: false
            permanent: true
        }
        ListElement {
            title: "Paul Hardy's Possible Tunebook"
            book: "pgh_possible_tunebook.abc"
            url: "http://www.pghardy.net/concertina/tunebooks/pgh_possible_tunebook.abc"
            downloaded: false
            permanent: true
        }
        ListElement {
            title: "Paul Hardy's Christmas Tunebook"
            book: "pgh_xmas_tunebook.abc"
            url: "http://www.pghardy.net/concertina/tunebooks/pgh_xmas_tunebook.abc"
            downloaded: false
            permanent: true
        }
//        ListElement {
//            title: "William Clarke of Feltwell Tunebook"
//            book: "williamclarke_tunes.abc"
//            url: "http://www.pghardy.net/concertina/tunebooks/williamclarke_tunes.abc"
//        downloaded: false
//        permanent: true
//        }
    }
    ListModel {
        id: booklistfilemodel
        function addFile(name) {
            booklistfilemodel.append({ "filename" :  name });
        }
    }


    Library {
        id:library
        signal clearAllSelected()
        signal downloadedChanged()
        property bool editMode: false
        onFileStateChanged: {
            switch (fileState) {
                case Library.None:
                case Library.Found:
                case Library.Loading:
                case Library.NotFound:
                case Library.Downloading:
                    //downloadsessionbook.text = "Downloading";
                    break;
                case Library.ShareSucceeded:
                case Library.ShareFailed:
                case Library.GetSharedLinksFailed:
                case Library.DownloadFailed:
                    clearAllSelected();
                    break;
                case Library.Downloaded:
                    clearAllSelected();
                    updateDownloaded(booklistmodel);
                    break;
                case Library.GetSharedLinksSucceeded:
                    clearAllSelected();
                    addSharedLinks();
                    break;
            }
        }
        onMessageChanged: {
            librarymessagetext.fade();
        }
        function addSharedLinks() {
            var toExclude = [];
            for (var i = 0; i < booklistmodel.count; i++) {
                var data = booklistmodel.get(i);
                toExclude.push(data.url);
            }
            var urls = getSharedUrls(toExclude);
            // now we have the urls that are not listed, we can add them
            for (i = 0; i < urls.length; i++) {
                var title = getSharedUrlTitle(urls[i]);
                var path = createPathFromUrl(urls[i]);
                booklistmodel.addTunebook(title, path, urls[i]);
            }
        }
        function updateDownloaded(model) {
            var changed = false;
            var i;
            for (i = 0; i < model.count; i++) {
                var data = model.get(i);
                var downloaded = isBookDownloaded(data.book);
                if (data.downloaded !== downloaded) {
                    data.downloaded = downloaded;
                    changed = true;
                }
            }
            if (changed)
                downloadedChanged(); // emit signal
        }
        Component.onCompleted: {
            booklistmodel.loadTunebooks();
            updateDownloaded(booklistmodel);
        }
    }
    Item {
        id: librarystate
        states: [
            State {
                name: "adding_entry"
            }

        ]
    }
    ColumnLayout {
        id: booklistcolumn
        anchors.fill: parent
        //anchors.centerIn: parent
        width: parent.width
        spacing: settings.liblist_spacing
        Heading {

        }
//        Rectangle {
//            height: 80
//        }
        BookList {
            id: liblist
            Layout.fillHeight: true
            Layout.alignment: Qt.AlignHCenter/* | Qt.AlignVCenter*/
            visible: !bookentryform.fileMode && !confirmdeletion.visible && booklistmodel.editingItem === -1
        }
        FileList {
            id: filelist
            Layout.fillHeight: true
            Layout.alignment: Qt.AlignHCenter/* | Qt.AlignVCenter*/
        }
        Disclaimer
        {
            id: disclaimer
            Layout.alignment: Qt.AlignHCenter
        }
        Confirmation {
            id: confirmdeletion
            Layout.alignment: Qt.AlignHCenter
        }

        BookEntry {
            id: bookentryform
            Layout.alignment: Qt.AlignHCenter
            Layout.fillHeight: !liblist.visible && !filelist.visible
        }

        ActionRow {
            Layout.alignment: Qt.AlignHCenter // | Qt.AlignBottom
            visible: !bookentryform.visible
        }
        ActionMessage {
            id: librarymessagetext
            Layout.alignment: Qt.AlignHCenter
        }

    }   // columnlayout
}
