function drawPart(part, ctx, x, top, bottom, xEnd) {
    ctx.save()
    //ctx.font = "9px sans-seri";
    ctx.beginPath();
    ctx.globalAlpha = 0.3;
    ctx.font = score.fontPart;
    ctx.text(part, x - 10, top - 20)
    ctx.stroke();
    ctx.fill();
    ctx.restore();
}


function drawRepeatEnding(number, ctx, start, top, end) {
    var x = start.x;
    var xEnd = end.x;
    if (top !== start.y) {
        // we have wrapped to the next line
        top = start.y;
    }
    // settings.score_scale
    var yLineBottom = top - 5 * settings.score_scale;
    var yLineTop = top - 13 * settings.score_scale;
    var xLineStart = xEnd - 10;
    var xText = x + 3;
    var yText = top - 6 * settings.score_scale;
    ctx.save()
    ctx.font = score.fontRepeatEnding;  // "8px sans-seri";
    ctx.stokeStyle = "black";
    ctx.lineWidth = 1;
    ctx.beginPath();
    ctx.moveTo(x, yLineBottom);
    ctx.lineTo(x, yLineTop);
    ctx.lineTo(xLineStart, yLineTop);
    ctx.stroke();

    ctx.beginPath();
    ctx.text(number.toString(), xText, yText)
    ctx.stroke();
    ctx.fill();
    ctx.restore();
}

function drawStartBar(bar, ctx, top, bottom) {
    var x = bar.xPosStart;
    var barStart = bar.start;
    if (barStart === "|:" || barStart === ":")
    {
        return openingRepeatBar(ctx, x, top, bottom);
    }
    if (barStart === "||")
    {
        return openingDoubleBar(ctx, x, top, bottom);
    }
    if (barStart !== "")
    {
        return openingBar(ctx, x, top, bottom);
    }
    return x;
}

function measureStartBar(barStart, x) {
    if (barStart === "|:" || barStart === ":")
    {
        return measureRepeatBar(x);
    }
    if (barStart === "||")
    {
        return measureDoubleBar(x);
    }
    if (barStart !== "")
    {
        return measureBar(x);
    }
    return x;
}
function thickBar(ctx, x, top, bottom) {
    ctx.stokeStyle = "black";
    ctx.lineWidth = 3;
    ctx.beginPath();
    ctx.moveTo(x, top);
    ctx.lineTo(x, bottom);
    ctx.stroke();
}

function thinBar(ctx, x, top, bottom) {
    ctx.stokeStyle = "black";
    ctx.lineWidth = 1;
    ctx.beginPath();
    ctx.moveTo(x, top);
    ctx.lineTo(x, bottom);
    ctx.stroke();
}
function repeatDots(ctx, x, top, bottom) {
    ctx.stokeStyle = "black";
    ctx.lineWidth = 1;
    ctx.beginPath();
    var size = 2 * settings.score_scale;
    var mid = -size / 2 + (top + bottom) / 2;
    var yTopDot = mid - score.spacing / 2;
    var yBottomDot = mid + score.spacing / 2;
//    var yOffset = 13.5;
//    var yTopDot = top + yOffset * settings.score_scale;
//    var yBottomDot = yTopDot + score.spacing; // top + (yOffset + 10) * settings.score_scale;
    ctx.ellipse(x, yTopDot, size, size);
    ctx.ellipse(x, yBottomDot, size, size);
    ctx.fill();
    ctx.stroke();
}

function openingRepeatBar(ctx, x, top, bottom) {
    thickBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    repeatDots(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}

function openingDoubleBar(ctx, x, top, bottom) {
    //var double_bar_spacing = 6;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function openingBar(ctx, x, top, bottom) {
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function drawEndBar(bar, ctx, top, bottom) {
    var x = bar.xPosEnd;
    var barEnd = bar.end;
    if (barEnd === ":|" || barEnd === ":")
    {
        return closingRepeatBar(ctx, x, top, bottom);
    }
    if (barEnd === ":||")
    {
        return closingRepeatDoubleBar(ctx, x, top, bottom);
    }
    if (barEnd === "|]")
    {
        return closingDoubleThickBar(ctx, x, top, bottom);
    }
    if (barEnd === "||")
    {
        return closingDoubleBar(ctx, x, top, bottom);
    }
    if (barEnd !== "")
    {
        return closingBar(ctx, x, top, bottom);
    }
    return x;
}
function measureEndBar(barEnd, x) {
    if (barEnd === ":|" || barEnd === ":")
    {
        return measureRepeatBar(x);
    }
    if (barEnd === ":||")
    {
        return measureRepeatDoubleBar(x);
    }
    if (barEnd === "|]")
    {
        return measureDoubleThickBar(x);
    }
    if (barEnd === "||")
    {
        return measureDoubleBar(x);
    }
    if (barEnd !== "")
    {
        return measureBar(x);
    }
    return x;
}
function closingBar(ctx, x, top, bottom) {
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function measureBar(x) {
    x += score.double_bar_spacing;
    return x;
}
function closingDoubleBar(ctx, x, top, bottom) {
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function measureDoubleBar(x) {
    x += score.double_bar_spacing;
    x += score.double_bar_spacing;
    return x;
}
function closingRepeatBar(ctx, x, top, bottom) {
    repeatDots(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thickBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function closingRepeatDoubleBar(ctx, x, top, bottom) {
    repeatDots(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    // next thin bar is done be next bar start
    return x;
}
function measureRepeatBar(x) {
    x += score.double_bar_spacing;
    x += score.double_bar_spacing;
    x += score.double_bar_spacing;
    return x;
}
function measureRepeatDoubleBar(x) {
    x += score.double_bar_spacing;
    x += score.double_bar_spacing;
    return x;
}
function closingDoubleThickBar(ctx, x, top, bottom) {
    thinBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    thickBar(ctx, x, top, bottom);
    x += score.double_bar_spacing;
    return x;
}
function measureDoubleThickBar(x) {
    x += score.double_bar_spacing;
    x += score.double_bar_spacing;
    return x;
}

function writeChord(name, ctx, x, y) {
    ctx.save();
    ctx.globalAlpha = 1.0
    ctx.beginPath();
    ctx.font = score.fontChord;   //"12px sans-seri";
    ctx.text(name, x, y)
    ctx.stroke();
    ctx.fill();
    ctx.restore();
}

function drawTupletNumber(note, ctx, x, top, number) {
    // TODO: use the note to determine the vertial position
    ctx.beginPath();
    ctx.font = score.fontTuplet;  //"10px sans-seri";
    ctx.text(number.toString(), x, top)
    ctx.stroke();
    ctx.fill();
}

function drawTupletNumberOnBeam(ctx, beamAttributes, number) {
    ctx.beginPath();
    ctx.font = score.fontTuplet;  //"10px sans-seri";
    var yAdjusted = beamAttributes.y;
    if (beamAttributes.up) {
        yAdjusted -= score.spacing;
    }
    else {
        yAdjusted += score.spacing;
    }
    ctx.text(number.toString(), beamAttributes.x, yAdjusted);
    ctx.stroke();
    ctx.fill();
}

function measureMeter(x, meter)
{
    if (meter !== null) {
        if (meter === "12/8")
            return x + 40 * settings.score_scale;
        return x + 25 * settings.score_scale;
    }
    return x;
}

function drawMeter(ctx, x, y, meter) {
    var xreturn = x;
    var scale_factor = 0.87 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 80 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 14 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);

    var xStep = 25;
    if (meter === "2/2" || meter === "C|")
        draw2_2(ctx, x, y)
    else if (meter === "3/4")
        draw3_4(ctx, x, y)
    else if (meter === "5/4")
        draw5_4(ctx, x, y)
    else if (meter === "6/8")
        draw6_8(ctx, x, y)
    else if (meter === "4/4" || meter === 'C')
        draw4_4(ctx, x, y)
    else if (meter === "3/8")
        draw3_8(ctx, x, y)
    else if (meter === "9/8")
        draw9_8(ctx, x, y)
    else if (meter === "3/2")
        draw3_2(ctx, x, y)
    else if (meter === "2/4")
        draw2_4(ctx, x, y)
    else if (meter === "4/2")
        draw4_2(ctx, x, y)
    else if (meter === "12/8") {
        draw12_8(ctx, x, y)
        xStep = 40;
    }
    else if (meter === "6/4")
        draw6_4(ctx, x, y);
    else
        console.exception("unsupported meter: " + meter);
        //console.assert(false, "unsupported meter: " + meter);
    ctx.resetTransform();
    return xreturn + xStep * settings.score_scale;
}

function draw12_8(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M82.561,45.553l3.598-3.796h4.793v15.278l0.104,0.756c0,0,0.155,0.392,0.473,0.481c0.316,0.09,0.107,0.112,0.724,0.149"
            + "s1.788,0.17,1.788,0.17v2.51H82.22v-2.639l1.204-0.004c0,0,0.974-0.07,1.078-0.091c0.104-0.02,0.94-0.059,0.981-1.058"
            + "s0.103-11.117,0.103-11.117l-1.119,1.139L82.561,45.553z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M92.275,76.212c-0.992-0.568-1.736-1.228-2.233-1.979c-0.496-0.749-0.744-1.595-0.744-2.536"
            + "c0-1.83,0.734-3.267,2.203-4.313c1.468-1.043,3.511-1.564,6.126-1.564c2.286,0,4.123,0.438,5.508,1.318"
            + "c1.385,0.877,2.078,2.026,2.078,3.442c0,0.859-0.27,1.603-0.806,2.234c-0.538,0.631-1.36,1.168-2.468,1.612"
            + "c1.459,0.518,2.55,1.231,3.273,2.141c0.725,0.911,1.086,2.017,1.086,3.32c0,1.996-0.82,3.565-2.458,4.707"
            + "c-1.64,1.144-3.912,1.714-6.818,1.714c-2.741,0-4.85-0.487-6.328-1.466c-1.479-0.977-2.219-2.364-2.219-4.164"
            + "c0-1.025,0.331-1.926,0.992-2.706C90.13,77.191,91.065,76.604,92.275,76.212z M94.727,77.608c-0.785,0.383-1.367,0.816-1.744,1.304"
            + "c-0.378,0.485-0.566,1.044-0.566,1.675c0,0.981,0.4,1.763,1.201,2.342c0.802,0.58,1.881,0.869,3.234,0.869"
            + "c1.469,0,2.553-0.207,3.25-0.62c0.697-0.414,1.047-1.056,1.047-1.925c0-0.547-0.374-1.057-1.123-1.528"
            + "C99.273,79.255,97.508,78.55,94.727,77.608z M99.426,73.31c0.621-0.319,1.085-0.696,1.39-1.131"
            + "c0.306-0.436,0.458-0.936,0.458-1.506c0-0.683-0.339-1.233-1.017-1.652c-0.678-0.418-1.591-0.627-2.737-0.627"
            + "c-1.023,0-1.826,0.176-2.404,0.527c-0.579,0.351-0.869,0.843-0.869,1.473c0,0.931,1.572,1.856,4.715,2.778L99.426,73.31z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M97.069,61.102c-0.021-0.124-0.033-0.259-0.038-0.404c-0.006-0.146-0.008-0.361-0.008-0.651"
            + "c0-1.188,0.245-2.28,0.737-3.272c0.49-0.992,1.253-1.945,2.287-2.854c0.723-0.631,1.871-1.385,3.443-2.265"
            + "c0.703-0.383,1.215-0.678,1.535-0.885c0.983-0.609,1.658-1.181,2.025-1.714c0.367-0.532,0.551-1.165,0.551-1.9"
            + "c0-0.909-0.345-1.642-1.032-2.194c-0.688-0.554-1.605-0.831-2.753-0.831c-0.941,0-1.723,0.187-2.342,0.56"
            + "c-0.621,0.373-1.076,0.915-1.365,1.629c0.186-0.041,0.363-0.078,0.535-0.108c0.17-0.031,0.317-0.048,0.441-0.048"
            + "c0.662,0,1.199,0.202,1.613,0.605c0.414,0.404,0.621,0.926,0.621,1.566c0,0.734-0.282,1.357-0.846,1.87"
            + "c-0.564,0.512-1.259,0.768-2.087,0.768c-1.024,0-1.841-0.314-2.45-0.946c-0.61-0.631-0.915-1.484-0.915-2.56"
            + "c0-1.778,0.783-3.265,2.35-4.459s3.55-1.792,5.949-1.792c2.554,0,4.594,0.584,6.118,1.753c1.525,1.169,2.289,2.726,2.289,4.668"
            + "c0,1.458-0.492,2.708-1.473,3.747c-0.984,1.038-2.438,1.844-4.359,2.411c-0.27,0.084-0.646,0.193-1.133,0.326"
            + "c-3.145,0.922-5.15,1.985-6.02,3.196c0.643-0.352,1.291-0.613,1.947-0.784c0.657-0.171,1.342-0.255,2.055-0.255"
            + "c0.775,0,1.697,0.116,2.762,0.348c1.065,0.234,1.753,0.351,2.063,0.351c0.651,0,1.116-0.207,1.396-0.622"
            + "c0.279-0.413,0.42-1.105,0.42-2.078h2.791c0.012,0.146,0.019,0.297,0.023,0.458c0.006,0.16,0.009,0.396,0.009,0.706"
            + "c0,2.13-0.466,3.702-1.396,4.715c-0.931,1.014-2.373,1.521-4.328,1.521c-0.94,0-2.109-0.284-3.505-0.853"
            + "c-0.229-0.094-0.394-0.16-0.497-0.201c-0.714-0.271-1.29-0.463-1.729-0.582c-0.44-0.12-0.814-0.179-1.125-0.179"
            + "c-0.434,0-0.81,0.104-1.125,0.311c-0.314,0.206-0.565,0.517-0.752,0.931H97.069z";
    ctx.fill();
    ctx.stroke();
}


function draw2_2(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M76.982,61.103c-0.021-0.124-0.033-0.259-0.039-0.404s-0.007-0.361-0.007-0.651"
            + "c0-1.188,0.245-2.28,0.737-3.272c0.491-0.992,1.253-1.945,2.288-2.854c0.724-0.631,1.872-1.385,3.444-2.265"
            + "c0.703-0.383,1.215-0.678,1.536-0.885c0.982-0.609,1.658-1.181,2.025-1.714c0.366-0.532,0.55-1.165,0.55-1.9"
            + "c0-0.909-0.344-1.642-1.031-2.194c-0.688-0.554-1.605-0.831-2.753-0.831c-0.941,0-1.722,0.187-2.342,0.56s-1.075,0.915-1.365,1.629"
            + "c0.187-0.041,0.365-0.078,0.536-0.108c0.17-0.031,0.318-0.048,0.442-0.048c0.662,0,1.2,0.202,1.613,0.605"
            + "c0.414,0.404,0.62,0.926,0.62,1.566c0,0.734-0.281,1.357-0.845,1.87c-0.564,0.512-1.259,0.768-2.086,0.768"
            + "c-1.023,0-1.841-0.314-2.451-0.946c-0.611-0.631-0.915-1.484-0.915-2.56c0-1.778,0.783-3.265,2.35-4.459s3.55-1.792,5.949-1.792"
            + "c2.555,0,4.594,0.584,6.119,1.753s2.289,2.726,2.289,4.668c0,1.458-0.492,2.708-1.475,3.747c-0.982,1.038-2.436,1.844-4.357,2.411"
            + "c-0.27,0.084-0.646,0.193-1.133,0.326c-3.144,0.922-5.15,1.985-6.02,3.196c0.642-0.352,1.291-0.613,1.947-0.784"
            + "c0.657-0.171,1.342-0.255,2.056-0.255c0.776,0,1.696,0.116,2.761,0.348c1.064,0.234,1.752,0.351,2.063,0.351"
            + "c0.651,0,1.116-0.207,1.396-0.622c0.279-0.413,0.418-1.105,0.418-2.078h2.793c0.01,0.146,0.018,0.297,0.023,0.458"
            + "c0.005,0.16,0.008,0.396,0.008,0.706c0,2.13-0.465,3.702-1.396,4.715c-0.93,1.014-2.373,1.521-4.327,1.521"
            + "c-0.941,0-2.11-0.284-3.506-0.853c-0.228-0.094-0.393-0.16-0.497-0.201c-0.713-0.271-1.29-0.463-1.729-0.582"
            + "c-0.439-0.12-0.814-0.179-1.125-0.179c-0.435,0-0.81,0.104-1.125,0.311c-0.316,0.206-0.566,0.517-0.752,0.931H76.982z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M76.982,85.733c-0.021-0.123-0.033-0.258-0.039-0.403c-0.005-0.145-0.007-0.362-0.007-0.651"
            + "c0-1.189,0.245-2.281,0.737-3.273c0.491-0.992,1.253-1.944,2.288-2.854c0.724-0.63,1.872-1.385,3.444-2.265"
            + "c0.703-0.383,1.215-0.677,1.536-0.885c0.982-0.609,1.658-1.181,2.025-1.713c0.366-0.533,0.55-1.166,0.55-1.9"
            + "c0-0.91-0.344-1.642-1.031-2.195c-0.688-0.554-1.605-0.831-2.753-0.831c-0.941,0-1.722,0.188-2.342,0.56"
            + "c-0.621,0.373-1.075,0.915-1.365,1.629c0.187-0.04,0.365-0.078,0.536-0.108c0.17-0.031,0.318-0.048,0.442-0.048"
            + "c0.662,0,1.2,0.203,1.613,0.605c0.414,0.404,0.62,0.927,0.62,1.567c0,0.734-0.281,1.357-0.845,1.869s-1.259,0.768-2.086,0.768"
            + "c-1.023,0-1.841-0.314-2.451-0.946c-0.611-0.631-0.915-1.483-0.915-2.56c0-1.778,0.783-3.265,2.35-4.459s3.55-1.791,5.949-1.791"
            + "c2.555,0,4.594,0.583,6.119,1.752c1.525,1.17,2.289,2.726,2.289,4.668c0,1.459-0.492,2.708-1.475,3.747s-2.436,1.845-4.357,2.413"
            + "c-0.27,0.082-0.646,0.191-1.133,0.324c-3.144,0.922-5.15,1.986-6.02,3.196c0.642-0.351,1.291-0.612,1.947-0.783"
            + "c0.657-0.17,1.342-0.255,2.056-0.255c0.776,0,1.696,0.115,2.761,0.348c1.064,0.233,1.752,0.351,2.063,0.351"
            + "c0.651,0,1.116-0.208,1.396-0.623c0.279-0.412,0.418-1.104,0.418-2.078h2.793c0.01,0.146,0.018,0.299,0.023,0.458"
            + "c0.005,0.16,0.008,0.396,0.008,0.706c0,2.131-0.465,3.702-1.396,4.715c-0.93,1.014-2.373,1.521-4.327,1.521"
            + "c-0.941,0-2.11-0.285-3.506-0.854c-0.228-0.093-0.393-0.16-0.497-0.201c-0.713-0.271-1.29-0.463-1.729-0.581"
            + "c-0.439-0.12-0.814-0.18-1.125-0.18c-0.435,0-0.81,0.104-1.125,0.311c-0.316,0.207-0.566,0.517-0.752,0.931H76.982z";
    ctx.fill();
    ctx.stroke();
}

function draw4_2(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M75.833,85.214c-0.021-0.124-0.034-0.259-0.038-0.404c-0.006-0.146-0.008-0.361-0.008-0.651"
            + "c0-1.188,0.245-2.28,0.737-3.272c0.49-0.992,1.253-1.945,2.288-2.854c0.723-0.631,1.871-1.385,3.444-2.265"
            + "c0.703-0.383,1.215-0.678,1.535-0.885c0.983-0.609,1.658-1.181,2.024-1.714c0.367-0.532,0.551-1.165,0.551-1.9"
            + "c0-0.909-0.344-1.642-1.031-2.194c-0.688-0.554-1.605-0.831-2.752-0.831c-0.942,0-1.723,0.187-2.343,0.56"
            + "c-0.621,0.373-1.075,0.915-1.365,1.629c0.186-0.041,0.364-0.078,0.535-0.108c0.17-0.031,0.318-0.048,0.442-0.048"
            + "c0.661,0,1.199,0.202,1.613,0.605c0.413,0.404,0.62,0.926,0.62,1.566c0,0.734-0.282,1.357-0.845,1.87"
            + "c-0.564,0.512-1.259,0.768-2.087,0.768c-1.024,0-1.84-0.314-2.45-0.946c-0.61-0.631-0.915-1.484-0.915-2.56"
            + "c0-1.778,0.783-3.265,2.35-4.459s3.549-1.792,5.949-1.792c2.553,0,4.594,0.584,6.119,1.753s2.288,2.726,2.288,4.668"
            + "c0,1.458-0.491,2.708-1.474,3.747c-0.982,1.038-2.436,1.844-4.359,2.411c-0.269,0.084-0.646,0.193-1.132,0.326"
            + "c-3.144,0.922-5.149,1.985-6.019,3.196c0.642-0.352,1.29-0.613,1.947-0.784c0.657-0.171,1.342-0.255,2.055-0.255"
            + "c0.775,0,1.696,0.116,2.761,0.348c1.065,0.234,1.753,0.351,2.063,0.351c0.651,0,1.117-0.207,1.396-0.622"
            + "c0.279-0.413,0.418-1.105,0.418-2.078h2.793c0.01,0.146,0.018,0.297,0.022,0.458c0.005,0.16,0.009,0.396,0.009,0.706"
            + "c0,2.13-0.467,3.702-1.396,4.715c-0.932,1.014-2.373,1.521-4.328,1.521c-0.941,0-2.109-0.284-3.505-0.853"
            + "c-0.228-0.094-0.393-0.16-0.497-0.201c-0.714-0.271-1.291-0.463-1.729-0.582c-0.44-0.12-0.814-0.179-1.125-0.179"
            + "c-0.434,0-0.809,0.104-1.124,0.311c-0.315,0.206-0.566,0.517-0.752,0.931H75.833z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M81.871,60.45v-2.638h0.854c1.003,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302H75.93V51.53l9.882-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
            + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638H81.871z M79.11,51.779h6.065l0.062-6.235L79.11,51.779z";
    ctx.fill();
    ctx.stroke();
}

function draw6_4(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M82.461,50.554c0.692-0.475,1.415-0.826,2.164-1.055c0.749-0.228,1.565-0.342,2.444-0.342"
        + "c2.078,0,3.725,0.55,4.939,1.646c1.216,1.097,1.824,2.58,1.824,4.451c0,1.903-0.748,3.449-2.242,4.638"
        + "c-1.495,1.19-3.447,1.784-5.856,1.784c-2.823,0-5.049-0.866-6.678-2.598c-1.628-1.732-2.442-4.109-2.442-7.128"
        + "c0-3.33,0.866-5.951,2.598-7.864c1.731-1.913,4.119-2.87,7.158-2.87c2.049,0,3.682,0.386,4.902,1.156"
        + "c1.221,0.77,1.83,1.781,1.83,3.031c0,0.808-0.276,1.464-0.829,1.971c-0.554,0.506-1.271,0.76-2.148,0.76"
        + "c-0.849,0-1.528-0.217-2.04-0.651s-0.768-1.008-0.768-1.722c0-0.414,0.104-0.771,0.31-1.07c0.208-0.301,0.517-0.548,0.931-0.744"
        + "c-0.278-0.104-0.554-0.182-0.822-0.233c-0.269-0.051-0.521-0.078-0.759-0.078c-1.479,0-2.601,0.554-3.367,1.66"
        + "c-0.765,1.106-1.148,2.73-1.148,4.871V50.554z M85.486,59.101c0.786,0,1.411-0.305,1.877-0.915c0.465-0.608,0.697-1.438,0.697-2.48"
        + "c0-1.232-0.23-2.181-0.689-2.849c-0.461-0.665-1.109-0.998-1.948-0.998c-0.796,0-1.448,0.327-1.955,0.983"
        + "c-0.506,0.657-0.76,1.513-0.76,2.567c0,1.189,0.241,2.103,0.723,2.737C83.911,58.783,84.596,59.101,85.486,59.101z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M82.219,85.733v-2.638h0.853c1.003,0,1.662-0.088,1.979-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
        + "v-1.302h-9.246v-3.041l9.881-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
        + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638H82.219z M79.458,77.063h6.065l0.062-6.235L79.458,77.063z";
//    ctx.path = "M81.871,85.733v-2.638h0.854c1.003,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
//            + "v-1.302H75.93v-3.041l9.882-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
//            + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638z M79.11,77.063h6.065l0.062-6.235L79.11,77.063z";
    ctx.fill();
    ctx.stroke();
}

function draw2_4(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M76.069,61.103c-0.021-0.124-0.034-0.259-0.038-0.404c-0.006-0.146-0.008-0.361-0.008-0.651"
            + "c0-1.188,0.245-2.28,0.737-3.272c0.49-0.992,1.253-1.945,2.288-2.854c0.723-0.631,1.871-1.385,3.444-2.265"
            + "c0.703-0.383,1.215-0.678,1.535-0.885c0.983-0.609,1.658-1.181,2.024-1.714c0.367-0.532,0.551-1.165,0.551-1.9"
            + "c0-0.909-0.344-1.642-1.031-2.194c-0.688-0.554-1.605-0.831-2.752-0.831c-0.942,0-1.723,0.187-2.343,0.56"
            + "c-0.621,0.373-1.075,0.915-1.365,1.629c0.186-0.041,0.364-0.078,0.535-0.108c0.17-0.031,0.318-0.048,0.442-0.048"
            + "c0.661,0,1.199,0.202,1.613,0.605c0.413,0.404,0.62,0.926,0.62,1.566c0,0.734-0.282,1.357-0.845,1.87"
            + "c-0.564,0.512-1.259,0.768-2.087,0.768c-1.024,0-1.84-0.314-2.45-0.946c-0.61-0.631-0.915-1.484-0.915-2.56"
            + "c0-1.778,0.783-3.265,2.35-4.459s3.549-1.792,5.949-1.792c2.553,0,4.594,0.584,6.119,1.753s2.288,2.726,2.288,4.668"
            + "c0,1.458-0.491,2.708-1.474,3.747c-0.982,1.038-2.436,1.844-4.359,2.411c-0.269,0.084-0.646,0.193-1.132,0.326"
            + "c-3.144,0.922-5.149,1.985-6.019,3.196c0.642-0.352,1.29-0.613,1.947-0.784c0.657-0.171,1.342-0.255,2.055-0.255"
            + "c0.775,0,1.696,0.116,2.761,0.348c1.065,0.234,1.753,0.351,2.063,0.351c0.651,0,1.117-0.207,1.396-0.622"
            + "c0.279-0.413,0.418-1.105,0.418-2.078h2.793c0.01,0.146,0.018,0.297,0.022,0.458c0.005,0.16,0.009,0.396,0.009,0.706"
            + "c0,2.13-0.467,3.702-1.396,4.715c-0.932,1.014-2.373,1.521-4.328,1.521c-0.941,0-2.109-0.284-3.505-0.853"
            + "c-0.228-0.094-0.393-0.16-0.497-0.201c-0.714-0.271-1.291-0.463-1.729-0.582c-0.44-0.12-0.814-0.179-1.125-0.179"
            + "c-0.434,0-0.809,0.104-1.124,0.311c-0.315,0.206-0.566,0.517-0.752,0.931H76.069z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M81.871,85.733v-2.638h0.854c1.003,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302H75.93v-3.041l9.882-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
            + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638z M79.11,77.063h6.065l0.062-6.235L79.11,77.063z";
    ctx.fill();
    ctx.stroke();
}
function draw3_2(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M83.106,52.308V49.64c1.302-0.063,2.272-0.329,2.908-0.8c0.637-0.471,0.954-1.149,0.954-2.04"
            + "c0-0.919-0.344-1.646-1.031-2.18c-0.688-0.532-1.621-0.798-2.8-0.798c-0.642,0-1.218,0.087-1.73,0.264"
            + "c-0.512,0.176-0.975,0.449-1.388,0.821h0.279c0.672,0,1.208,0.187,1.606,0.559c0.398,0.373,0.597,0.874,0.597,1.505"
            + "c0,0.755-0.249,1.359-0.746,1.815c-0.496,0.454-1.157,0.683-1.985,0.683c-0.91,0-1.626-0.261-2.148-0.783"
            + "c-0.522-0.523-0.784-1.244-0.784-2.166c0-1.612,0.691-2.899,2.071-3.861c1.381-0.962,3.254-1.442,5.623-1.442"
            + "c2.626,0,4.648,0.452,6.065,1.357s2.125,2.19,2.125,3.854c0,1.096-0.358,2.02-1.078,2.77c-0.719,0.749-1.713,1.231-2.985,1.45"
            + "c1.634,0.413,2.851,1.027,3.653,1.838c0.801,0.813,1.201,1.832,1.201,3.063c0,2.007-0.798,3.535-2.396,4.584"
            + "c-1.598,1.051-3.932,1.575-7.003,1.575c-2.44,0-4.37-0.479-5.786-1.436c-1.417-0.956-2.125-2.246-2.125-3.869"
            + "c0-1.034,0.302-1.87,0.907-2.506s1.393-0.954,2.365-0.954c0.807,0,1.479,0.253,2.017,0.76c0.538,0.508,0.807,1.145,0.807,1.909"
            + "c0,0.703-0.23,1.264-0.69,1.682c-0.461,0.42-1.089,0.629-1.885,0.629c0.456,0.363,0.967,0.629,1.536,0.8"
            + "c0.568,0.171,1.225,0.255,1.97,0.255c1.344,0,2.378-0.313,3.102-0.938c0.724-0.624,1.086-1.521,1.086-2.69"
            + "c0-1.045-0.35-1.813-1.048-2.303C85.673,52.553,84.585,52.308,83.106,52.308z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M76.156,85.733c-0.021-0.123-0.033-0.258-0.039-0.403c-0.005-0.145-0.007-0.362-0.007-0.651"
            + "c0-1.189,0.245-2.281,0.737-3.273c0.491-0.992,1.253-1.944,2.288-2.854c0.724-0.63,1.872-1.385,3.444-2.265"
            + "c0.702-0.383,1.215-0.677,1.535-0.885c0.982-0.609,1.657-1.181,2.025-1.713c0.367-0.533,0.551-1.166,0.551-1.9"
            + "c0-0.91-0.344-1.642-1.032-2.195c-0.688-0.554-1.605-0.831-2.753-0.831c-0.941,0-1.722,0.188-2.343,0.56"
            + "c-0.62,0.373-1.075,0.915-1.365,1.629c0.187-0.04,0.365-0.078,0.536-0.108c0.169-0.031,0.317-0.048,0.441-0.048"
            + "c0.662,0,1.199,0.203,1.613,0.605c0.414,0.404,0.621,0.927,0.621,1.567c0,0.734-0.282,1.357-0.845,1.869"
            + "c-0.564,0.512-1.26,0.768-2.087,0.768c-1.023,0-1.84-0.314-2.451-0.946c-0.611-0.631-0.915-1.483-0.915-2.56"
            + "c0-1.778,0.783-3.265,2.35-4.459c1.567-1.194,3.55-1.791,5.949-1.791c2.555,0,4.594,0.583,6.119,1.752"
            + "c1.525,1.17,2.289,2.726,2.289,4.668c0,1.459-0.491,2.708-1.474,3.747c-0.983,1.039-2.436,1.845-4.358,2.413"
            + "c-0.27,0.082-0.646,0.191-1.133,0.324c-3.144,0.922-5.15,1.986-6.019,3.196c0.642-0.351,1.291-0.612,1.947-0.783"
            + "c0.656-0.17,1.341-0.255,2.055-0.255c0.776,0,1.696,0.115,2.761,0.348c1.065,0.233,1.753,0.351,2.063,0.351"
            + "c0.651,0,1.116-0.208,1.396-0.623c0.279-0.412,0.418-1.104,0.418-2.078h2.793c0.011,0.146,0.018,0.299,0.023,0.458"
            + "c0.005,0.16,0.008,0.396,0.008,0.706c0,2.131-0.465,3.702-1.396,4.715c-0.93,1.014-2.373,1.521-4.326,1.521"
            + "c-0.941,0-2.111-0.285-3.507-0.854c-0.228-0.093-0.393-0.16-0.497-0.201c-0.713-0.271-1.29-0.463-1.729-0.581"
            + "c-0.439-0.12-0.814-0.18-1.125-0.18c-0.435,0-0.81,0.104-1.125,0.311c-0.316,0.207-0.566,0.517-0.753,0.931H76.156z";
    ctx.fill();
    ctx.stroke();
}
function draw9_8(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M88.07,52.308c-0.704,0.485-1.433,0.846-2.187,1.077c-0.755,0.232-1.572,0.35-2.452,0.35"
            + "c-2.068,0-3.712-0.549-4.932-1.646c-1.222-1.095-1.832-2.58-1.832-4.452c0-1.923,0.748-3.479,2.242-4.668"
            + "c1.495-1.189,3.447-1.784,5.856-1.784c2.833,0,5.062,0.867,6.686,2.599s2.435,4.108,2.435,7.128c0,3.34-0.867,5.969-2.605,7.887"
            + "c-1.737,1.919-4.121,2.878-7.151,2.878c-2.038,0-3.668-0.39-4.894-1.17c-1.226-0.781-1.838-1.803-1.838-3.064"
            + "c0-0.797,0.28-1.45,0.838-1.962c0.558-0.513,1.288-0.769,2.187-0.769c0.837,0,1.507,0.223,2.01,0.667"
            + "c0.501,0.445,0.751,1.029,0.751,1.753c0,0.404-0.101,0.752-0.302,1.047c-0.202,0.296-0.516,0.54-0.939,0.736"
            + "c0.332,0.115,0.623,0.199,0.876,0.257s0.494,0.085,0.722,0.085c1.478,0,2.603-0.558,3.373-1.675"
            + "c0.771-1.116,1.156-2.752,1.156-4.902V52.308z M85.03,43.76c-0.787,0-1.412,0.306-1.877,0.915c-0.466,0.61-0.698,1.437-0.698,2.481"
            + "c0,1.231,0.229,2.186,0.691,2.862c0.459,0.678,1.109,1.016,1.946,1.016c0.796,0,1.449-0.335,1.961-1.008s0.769-1.535,0.769-2.591"
            + "c0-1.169-0.244-2.072-0.729-2.714C86.607,44.08,85.918,43.76,85.03,43.76z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M80.159,76.178c-0.992-0.568-1.737-1.228-2.233-1.979c-0.497-0.749-0.745-1.595-0.745-2.536"
            + "c0-1.83,0.734-3.267,2.204-4.313c1.467-1.043,3.51-1.564,6.126-1.564c2.285,0,4.123,0.438,5.507,1.318"
            + "c1.386,0.877,2.078,2.026,2.078,3.442c0,0.859-0.269,1.603-0.806,2.234c-0.538,0.631-1.36,1.168-2.468,1.612"
            + "c1.459,0.518,2.55,1.231,3.273,2.141c0.725,0.911,1.087,2.017,1.087,3.32c0,1.996-0.819,3.565-2.46,4.707"
            + "c-1.639,1.144-3.91,1.714-6.816,1.714c-2.741,0-4.851-0.487-6.329-1.466c-1.48-0.977-2.22-2.364-2.22-4.164"
            + "c0-1.025,0.332-1.926,0.994-2.706C78.012,77.157,78.949,76.57,80.159,76.178z M82.61,77.574c-0.786,0.383-1.368,0.816-1.746,1.304"
            + "c-0.376,0.485-0.565,1.044-0.565,1.675c0,0.981,0.4,1.763,1.202,2.342c0.802,0.58,1.88,0.869,3.235,0.869"
            + "c1.467,0,2.551-0.207,3.25-0.62c0.697-0.414,1.046-1.056,1.046-1.925c0-0.547-0.373-1.057-1.124-1.528"
            + "C87.157,79.221,85.392,78.516,82.61,77.574z M87.31,73.276c0.622-0.319,1.085-0.696,1.39-1.131"
            + "c0.305-0.436,0.457-0.936,0.457-1.506c0-0.683-0.339-1.233-1.017-1.652c-0.677-0.418-1.589-0.627-2.737-0.627"
            + "c-1.024,0-1.826,0.176-2.406,0.527c-0.578,0.351-0.868,0.843-0.868,1.473c0,0.931,1.572,1.856,4.715,2.778L87.31,73.276z";
    ctx.fill();
    ctx.stroke();
}
function draw3_8(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M83.193,52.308V49.64c1.303-0.063,2.272-0.329,2.909-0.8c0.636-0.471,0.953-1.149,0.953-2.04"
        + "c0-0.919-0.344-1.646-1.031-2.18c-0.688-0.532-1.621-0.798-2.8-0.798c-0.641,0-1.217,0.087-1.729,0.264"
        + "c-0.513,0.176-0.975,0.449-1.389,0.821h0.279c0.672,0,1.207,0.187,1.605,0.559c0.398,0.373,0.597,0.874,0.597,1.505"
        + "c0,0.755-0.248,1.359-0.745,1.815c-0.496,0.454-1.158,0.683-1.985,0.683c-0.91,0-1.626-0.261-2.149-0.783"
        + "c-0.522-0.523-0.783-1.244-0.783-2.166c0-1.612,0.69-2.899,2.071-3.861s3.255-1.442,5.623-1.442c2.626,0,4.649,0.452,6.066,1.357"
        + "c1.416,0.905,2.125,2.19,2.125,3.854c0,1.096-0.359,2.02-1.078,2.77c-0.719,0.749-1.715,1.231-2.986,1.45"
        + "c1.634,0.413,2.852,1.027,3.653,1.838c0.802,0.813,1.202,1.832,1.202,3.063c0,2.007-0.799,3.535-2.396,4.584"
        + "c-1.598,1.051-3.933,1.575-7.004,1.575c-2.44,0-4.369-0.479-5.786-1.436c-1.417-0.956-2.125-2.246-2.125-3.869"
        + "c0-1.034,0.303-1.87,0.908-2.506s1.394-0.954,2.366-0.954c0.807,0,1.479,0.253,2.017,0.76c0.538,0.508,0.807,1.145,0.807,1.909"
        + "c0,0.703-0.23,1.264-0.69,1.682c-0.46,0.42-1.089,0.629-1.885,0.629c0.455,0.363,0.966,0.629,1.536,0.8"
        + "c0.569,0.171,1.225,0.255,1.97,0.255c1.344,0,2.377-0.313,3.103-0.938c0.723-0.624,1.086-1.521,1.086-2.69"
        + "c0-1.045-0.35-1.813-1.048-2.303C85.76,52.553,84.671,52.308,83.193,52.308z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M82.045,85.733v-2.638h0.854c1.002,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
        + "v-1.302h-9.245v-3.041l9.881-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
        + "c0.284,0.176,0.892,0.264,1.822,0.264h0.838v2.638z M79.284,77.063h6.065l0.062-6.235L79.284,77.063z";
    ctx.fill();
    ctx.stroke();
}
function draw4_4(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M82.219,61.103v-2.638h0.853c1.003,0,1.662-0.088,1.979-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302h-9.246v-3.042l9.881-10.424h4.793v10.673h3.087v2.793h-3.087v1.907c0,0.538,0.142,0.895,0.427,1.07"
            + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638z M79.458,52.431h6.065l0.062-6.235L79.458,52.431z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M82.219,85.733v-2.638h0.853c1.003,0,1.662-0.088,1.979-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302h-9.246v-3.041l9.881-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
            + "c0.284,0.176,0.893,0.264,1.822,0.264h0.838v2.638z M79.458,77.063h6.065l0.062-6.235L79.458,77.063z";
    ctx.fill();
    ctx.stroke();
}
function draw6_8(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M82.461,50.554c0.692-0.475,1.415-0.826,2.164-1.055c0.749-0.228,1.565-0.342,2.444-0.342"
            + "c2.078,0,3.725,0.55,4.939,1.646c1.216,1.097,1.824,2.58,1.824,4.451c0,1.903-0.748,3.449-2.242,4.638"
            + "c-1.495,1.19-3.447,1.784-5.856,1.784c-2.823,0-5.049-0.866-6.678-2.598c-1.628-1.732-2.442-4.109-2.442-7.128"
            + "c0-3.33,0.866-5.951,2.598-7.864c1.731-1.913,4.119-2.87,7.158-2.87c2.049,0,3.682,0.386,4.902,1.156"
            + "c1.221,0.77,1.83,1.781,1.83,3.031c0,0.808-0.276,1.464-0.829,1.971c-0.554,0.506-1.271,0.76-2.148,0.76"
            + "c-0.849,0-1.528-0.217-2.04-0.651s-0.768-1.008-0.768-1.722c0-0.414,0.104-0.771,0.31-1.07c0.208-0.301,0.517-0.548,0.931-0.744"
            + "c-0.278-0.104-0.554-0.182-0.822-0.233c-0.269-0.051-0.521-0.078-0.759-0.078c-1.479,0-2.601,0.554-3.367,1.66"
            + "c-0.765,1.106-1.148,2.73-1.148,4.871V50.554z M85.486,59.101c0.786,0,1.411-0.305,1.877-0.915c0.465-0.608,0.697-1.438,0.697-2.48"
            + "c0-1.232-0.23-2.181-0.689-2.849c-0.461-0.665-1.109-0.998-1.948-0.998c-0.796,0-1.448,0.327-1.955,0.983"
            + "c-0.506,0.657-0.76,1.513-0.76,2.567c0,1.189,0.241,2.103,0.723,2.737C83.911,58.783,84.596,59.101,85.486,59.101z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M80.072,76.178c-0.993-0.568-1.737-1.228-2.233-1.979c-0.497-0.749-0.745-1.595-0.745-2.536"
            + "c0-1.83,0.734-3.267,2.203-4.313c1.468-1.043,3.511-1.564,6.127-1.564c2.286,0,4.122,0.438,5.507,1.318"
            + "c1.386,0.877,2.078,2.026,2.078,3.442c0,0.859-0.27,1.603-0.806,2.234c-0.539,0.631-1.361,1.168-2.468,1.612"
            + "c1.458,0.518,2.55,1.231,3.273,2.141c0.725,0.911,1.086,2.017,1.086,3.32c0,1.996-0.819,3.565-2.458,4.707"
            + "c-1.641,1.144-3.912,1.714-6.818,1.714c-2.741,0-4.85-0.487-6.329-1.466c-1.479-0.977-2.218-2.364-2.218-4.164"
            + "c0-1.025,0.331-1.926,0.992-2.706C77.926,77.157,78.862,76.57,80.072,76.178z M82.522,77.574c-0.786,0.383-1.367,0.816-1.745,1.304"
            + "c-0.377,0.485-0.566,1.044-0.566,1.675c0,0.981,0.4,1.763,1.201,2.342c0.802,0.58,1.88,0.869,3.235,0.869"
            + "c1.468,0,2.551-0.207,3.25-0.62c0.697-0.414,1.047-1.056,1.047-1.925c0-0.547-0.375-1.057-1.125-1.528"
            + "C87.07,79.221,85.304,78.516,82.522,77.574z M87.223,73.276c0.621-0.319,1.084-0.696,1.389-1.131"
            + "c0.305-0.436,0.457-0.936,0.457-1.506c0-0.683-0.338-1.233-1.016-1.652c-0.678-0.418-1.59-0.627-2.737-0.627"
            + "c-1.024,0-1.826,0.176-2.405,0.527c-0.579,0.351-0.869,0.843-0.869,1.473c0,0.931,1.572,1.856,4.716,2.778L87.223,73.276z";
    ctx.fill();
    ctx.stroke();
}
function draw3_4(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M83.193,52.308V49.64c1.303-0.063,2.272-0.329,2.909-0.8c0.636-0.471,0.953-1.149,0.953-2.04"
            + "c0-0.919-0.344-1.646-1.031-2.18c-0.688-0.532-1.621-0.798-2.8-0.798c-0.641,0-1.217,0.087-1.729,0.264"
            + "c-0.513,0.176-0.975,0.449-1.389,0.821h0.279c0.672,0,1.207,0.187,1.605,0.559c0.398,0.373,0.597,0.874,0.597,1.505"
            + "c0,0.755-0.248,1.359-0.745,1.815c-0.496,0.454-1.158,0.683-1.985,0.683c-0.91,0-1.626-0.261-2.149-0.783"
            + "c-0.522-0.523-0.783-1.244-0.783-2.166c0-1.612,0.69-2.899,2.071-3.861s3.255-1.442,5.623-1.442c2.626,0,4.649,0.452,6.066,1.357"
            + "c1.416,0.905,2.125,2.19,2.125,3.854c0,1.096-0.359,2.02-1.078,2.77c-0.719,0.749-1.715,1.231-2.986,1.45"
            + "c1.634,0.413,2.852,1.027,3.653,1.838c0.802,0.813,1.202,1.832,1.202,3.063c0,2.007-0.799,3.535-2.396,4.584"
            + "c-1.598,1.051-3.933,1.575-7.004,1.575c-2.44,0-4.369-0.479-5.786-1.436c-1.417-0.956-2.125-2.246-2.125-3.869"
            + "c0-1.034,0.303-1.87,0.908-2.506s1.394-0.954,2.366-0.954c0.807,0,1.479,0.253,2.017,0.76c0.538,0.508,0.807,1.145,0.807,1.909"
            + "c0,0.703-0.23,1.264-0.69,1.682c-0.46,0.42-1.089,0.629-1.885,0.629c0.455,0.363,0.966,0.629,1.536,0.8"
            + "c0.569,0.171,1.225,0.255,1.97,0.255c1.344,0,2.377-0.313,3.103-0.938c0.723-0.624,1.086-1.521,1.086-2.69"
            + "c0-1.045-0.35-1.813-1.048-2.303C85.76,52.553,84.671,52.308,83.193,52.308z";
    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M82.045,85.733v-2.638h0.854c1.002,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302h-9.245v-3.041l9.881-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
            + "c0.284,0.176,0.892,0.264,1.822,0.264h0.838v2.638z M79.284,77.063h6.065l0.062-6.235L79.284,77.063z";
    ctx.fill();
    ctx.stroke();
}

function draw5_4(ctx, x, y) {
    ctx.beginPath();
    ctx.path = "M77.821,58.032c0-0.793,0.257-1.487,0.771-2.081c0.517-0.596,1.159-0.893,1.924-0.893c1.878,0,2.818,0.859,2.818,2.578"
    + "c0,0.318-0.075,0.635-0.218,0.952c-0.149,0.318-0.306,0.615-0.478,0.892c-0.174,0.278-0.335,0.517-0.496,0.714"
    + "c-0.158,0.197-0.24,0.326-0.24,0.378c0.427,0.554,1.111,0.832,2.066,0.832c2.221,0,3.33-1.704,3.33-5.115"
    + "c0-1.349-0.277-2.44-0.833-3.273c-0.554-0.833-1.348-1.249-2.382-1.249c-1.003,0-2.085,0.343-3.25,1.031"
    + "c-0.157,0.079-0.251,0.151-0.276,0.218c-0.029,0.066-0.088,0.119-0.178,0.158c-0.094,0.041-0.252,0.081-0.478,0.12"
    + "c-0.226,0.041-0.587,0.1-1.092,0.179c0.082-1.057,0.181-2.38,0.298-3.967c0.119-1.586,0.244-3.45,0.378-5.591"
    + "c1.72,0.423,4.043,0.634,6.979,0.634c1.802,0,3.597-0.158,5.396-0.476c-0.322,2.83-2.821,4.244-7.499,4.244"
    + "c-0.131,0-0.35-0.013-0.652-0.041c-0.304-0.026-0.63-0.052-0.975-0.079c-0.343-0.027-0.666-0.054-0.97-0.08"
    + "c-0.303-0.026-0.535-0.04-0.696-0.04c-0.051,0-0.091,0.12-0.115,0.358c-0.026,0.237-0.059,0.522-0.102,0.852"
    + "c-0.041,0.331-0.067,0.688-0.08,1.071c-0.013,0.383-0.019,0.72-0.019,1.011c1.51-0.688,3.027-1.031,4.563-1.031"
    + "c0.974,0,1.903,0.139,2.775,0.416c0.872,0.278,1.639,0.675,2.297,1.191c0.666,0.515,1.175,1.136,1.55,1.863"
    + "c0.367,0.727,0.555,1.54,0.555,2.438c0,0.925-0.2,1.772-0.594,2.539c-0.4,0.765-0.948,1.414-1.646,1.943"
    + "c-0.702,0.529-1.523,0.938-2.458,1.229c-0.943,0.291-1.966,0.436-3.076,0.436c-1.085,0-2.049-0.126-2.896-0.377"
    + "c-0.845-0.252-1.567-0.575-2.162-0.969c-0.592-0.4-1.048-0.862-1.367-1.387C77.979,59.131,77.821,58.588,77.821,58.032z";

    ctx.fill();
    ctx.stroke();
    ctx.beginPath();
    ctx.path = "M82.045,85.733v-2.638h0.854c1.002,0,1.662-0.088,1.978-0.264c0.315-0.176,0.473-0.692,0.473-1.551v-0.125"
            + "v-1.302h-9.245v-3.041l9.881-10.424h4.793v10.673h3.087v2.792h-3.087v1.908c0,0.537,0.142,0.894,0.427,1.069"
            + "c0.284,0.176,0.892,0.264,1.822,0.264h0.838v2.638z M79.284,77.063h6.065l0.062-6.235L79.284,77.063z";
    ctx.fill();
    ctx.stroke();
}

function drawDotAbove(ctx, x, y) {
    var scale_factor = 0.4 * settings.score_scale;
    var size = 5 * scale_factor;
    x += 20 * scale_factor;
    y -= 34 * scale_factor;
    ctx.save();
    ctx.beginPath();
    ctx.ellipse(x - size / 2, y - size / 2, size, size);
    ctx.fill();
    ctx.restore();
}

function drawHat(ctx, x, top) {
    var scale_factor = 0.4 * settings.score_scale;
    var radius = 20.0 * scale_factor;
    var phi1 = -Math.PI;
    var phi2 = 0;   //Math.PI / 2;
    var y = top - 20 * scale_factor;
    x += 10 * scale_factor;
    var size = 10 * scale_factor;
    ctx.save();
    ctx.lineWidth = .5;
    ctx.beginPath();
    ctx.arc(x, y, radius, phi1, phi2);
    ctx.arc(x, y - size / 2, radius, phi2, phi1, true);
    ctx.stroke();
    ctx.fill();
    // now the dot
    ctx.beginPath();
    ctx.ellipse(x - size / 2, y - size / 2, size, size);
    ctx.fill();
    ctx.restore();
}

function drawWholeNote(ctx, x, y) {
    var scale_factor = 0.4 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 20 * settings.score_scale;
    xadj += 14 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M84.54,49.7c7.103-0.197,20.138,4.377,20.41,14.044c0.314,9.425-12.973,14.191-19.878,14.044"
        + "c-7.298,0.187-20.137-4.38-20.411-14.044C64.349,54.417,77.734,49.55,84.54,49.7z M87.199,75.379"
        + "c4.228-0.752,5.875-2.91,5.578-9.287c-0.488-8.731-5.444-15.011-10.364-14.048c-4.132,0.858-5.874,2.681-5.58,9.058"
        + "C77.224,69.83,82.279,76.338,87.199,75.379z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
}

function drawBareHalfNote(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 25 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M74.639,40.577c0.482-5.851,7.523-12.051,13.16-11.697c1.692,0.073,3.4,0.541,4.747,1.586"
        + "c2.157,1.692,2.616,3.975,2.551,6.458c-0.188,5.883-7.32,10.39-13.16,10.805c-1.693,0.107-3.709,0.454-5.48-1.252"
        + "C73.907,44.02,74.501,42.249,74.639,40.577z M76.81,43.547c0.001,0.957,0.472,1.261,1.618,1.404c1.959,0.024,5.621-1.682,8.67-4.024"
        + "c3.117-2.303,6.018-5.281,5.814-7.403c-0.066-0.757-0.742-1.313-1.419-1.441c-2.902-0.895-8.195,2.814-10.292,4.685"
        + "C78.628,39.02,76.676,41.59,76.81,43.547z";

    // TODO: image without stem:
//    ctx.path = "M93.345,28.541c0.606,0.078,1.212,0.166,1.816,0.261c0,19.058,0,38.116,0,57.174c-0.12,5.868-7.523,12.05-13.158,11.696"
//        + "c-1.695-0.072-3.404-0.449-4.751-1.491c-2.157-1.695-2.615-4.068-2.549-6.552c0.189-5.884,7.321-10.204,13.161-10.617"
//        + "c1.691-0.108,3.39,0.021,5.48,1.249C93.345,63.021,93.345,45.78,93.345,28.541z M76.887,93.005c0,0.95,0.47,1.267,1.617,1.578"
//        + "c1.959,0.341,5.619-1.084,8.669-3.614c3.12-2.44,6.01-6.018,5.806-8.104c-0.066-0.744-0.731-1.149-1.407-1.181"
//            + "c-2.902-0.363-8.196,3.419-10.294,5.025C78.703,88.758,76.751,91.013,76.887,93.005z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    xadj += 4 * settings.score_scale;
    yadj -= 26 * settings.score_scale;
    note.xBareNote = xadj;
    note.yBareNote = yadj;
}


function drawHalfNoteUp(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M93.345,28.541c0.606,0.078,1.212,0.166,1.816,0.261c0,19.058,0,38.116,0,57.174c-0.12,5.868-7.523,12.05-13.158,11.696"
        + "c-1.695-0.072-3.404-0.449-4.751-1.491c-2.157-1.695-2.615-4.068-2.549-6.552c0.189-5.884,7.321-10.204,13.161-10.617"
        + "c1.691-0.108,3.39,0.021,5.48,1.249C93.345,63.021,93.345,45.78,93.345,28.541z M76.887,93.005c0,0.95,0.47,1.267,1.617,1.578"
        + "c1.959,0.341,5.619-1.084,8.669-3.614c3.12-2.44,6.01-6.018,5.806-8.104c-0.066-0.744-0.731-1.149-1.407-1.181"
            + "c-2.902-0.363-8.196,3.419-10.294,5.025C78.703,88.758,76.751,91.013,76.887,93.005z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawHalfNoteDown(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 0 * settings.score_scale;
    yadj += 26 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M76.456,98.195c-0.606-0.079-1.213-0.166-1.817-0.262c0-19.119,0-38.236,0-57.356c0.122-5.869,7.523-12.05,13.16-11.697"
        + "c1.692,0.073,3.4,0.541,4.747,1.586c2.157,1.692,2.616,3.974,2.551,6.458c-0.189,5.882-7.321,10.389-13.16,10.804"
        + "c-1.693,0.108-3.391-0.023-5.48-1.252C76.456,63.715,76.456,80.954,76.456,98.195z M76.81,43.547"
        + "c0.001,0.957,0.472,1.26,1.618,1.404c1.959,0.025,5.621-1.681,8.67-4.024c3.119-2.303,6.018-5.282,5.816-7.404"
        + "c-0.068-0.757-0.742-1.312-1.419-1.442c-2.902-0.894-8.195,2.815-10.292,4.684C78.628,39.02,76.676,41.59,76.81,43.547z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    yadj -= 26 * settings.score_scale;
    note.yBareNote = yadj;
}

function drawThirtySecondNoteUp(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    xadj += 7 * settings.score_scale;
    var yadj = y - 51.5 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M86.367,92.471c0,3.152-1.873,6.059-4.517,8.104c-2.574,2.052-5.962,3.113-8.933,2.807"
        + "c-1.894-0.189-3.644-1.035-4.854-2.22c-1.28-1.197-2.019-2.792-2.019-4.547c0-2.767,1.881-5.3,4.442-7.166"
        + "c2.562-1.923,5.81-3.167,8.518-3.096c2.37,0.065,4.31,0.378,5.53,1.572c0-21.431,0-42.864,0-64.296"
        + "c0.611-0.001,1.222,0.001,1.832,0.009c0.135,2.533,0.501,6.094,3.414,8.909c3.321,3.116,6.362,6.295,8.454,9.109"
        + "c2.767,3.697,4.716,8.157,4.716,12.553c0,2.237-0.337,4.395-0.942,6.498c0.605,1.708,0.942,3.537,0.942,5.299"
        + "c0,1.763-0.202,3.528-0.606,5.103c1.145,2.141,1.411,4.147,1.411,7.123c0,4.724-1.36,8.771-3.314,12.233"
        + "c-0.363,0.031-0.727,0.063-1.092,0.093c1.416-2.323,3.063-7.638,2.928-10.884c-0.133-3.6-0.268-5.543-2.559-8.361"
        + "c-1.279-1.613-3.508-3.377-6.214-5.176c-2.638-1.871-5.239-3.809-7.138-5.144C86.367,71.486,86.367,81.979,86.367,92.471z"
         + "M101.201,54.829c-0.136-3.616-1.885-6.498-4.181-9.512c-2.633-3.444-6.857-6.514-10.653-8.784"
        + "c0.135,1.828,0.501,5.413,3.414,8.194c3.321,3.077,6.362,6.172,8.454,8.916c1.148,1.48,2.195,3.058,2.936,4.846"
        + "C101.237,57.192,101.269,55.854,101.201,54.829z M101.201,66.702c-0.136-3.61-1.885-6.417-4.181-9.361"
        + "c-2.633-3.359-6.857-6.222-10.653-8.482c0.135,3.016,2.06,7.482,4.563,9.736c3.117,2.82,6.838,6.519,9.462,9.553"
        + "c0.337,0.405,0.577,0.817,0.778,1.224C101.181,68.48,101.192,67.592,101.201,66.702z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawThirtySecondNoteDown(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    xadj += 2 * settings.score_scale;
    var yadj = y - 20.5 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M77.057,67.085c1.885-1.269,4.442-3.162,7.087-5.027c2.712-1.796,4.947-3.581,6.232-5.209"
        + "c2.299-2.846,2.432-4.801,2.567-8.414c0.135-3.26-1.553-8.553-2.229-9.346c0.306-0.208,0.613-0.414,0.92-0.619"
        + "c1.484,2.629,2.791,6.65,2.791,11.395c0,3.056-0.27,5.075-1.415,7.096c0.406,1.645,0.606,3.417,0.606,5.186"
        + "c0,1.77-0.337,3.539-0.944,5.25c0.607,2.044,0.945,4.348,0.944,6.594c0,4.412-1.952,8.891-4.729,12.494"
        + "c-2.099,2.744-5.154,5.768-8.469,8.514c-2.906,2.474-3.226,5.66-3.361,8.104c-0.546-0.072-1.092-0.151-1.637-0.237"
        + "c0-22.406,0-44.812,0-67.219c0-3.096,1.753-6.037,4.322-8.245c2.636-2.282,5.961-3.628,8.941-3.466"
        + "c1.965,0.096,3.653,0.893,4.865,2.062c1.28,1.178,2.021,2.778,2.021,4.531c0,2.768-1.881,5.296-4.449,7.211"
        + "c-2.502,1.976-5.759,3.317-8.536,3.393c-2.371,0.061-4.316-0.187-5.527-1.246C77.057,48.952,77.057,58.019,77.057,67.085z"
         + "M91.053,60.01c-2.636,3.06-6.37,6.755-9.485,9.477c-2.504,2.189-4.433,6.488-4.565,9.395c3.771-1.973,8.023-4.746,10.666-8.074"
        + "c2.303-2.911,4.061-5.732,4.194-9.351c-0.016-0.817-0.03-1.633-0.046-2.449C91.612,59.347,91.391,59.602,91.053,60.01z"
         + "M88.887,74.514c-2.099,2.709-5.154,5.721-8.469,8.545c-2.906,2.553-3.281,5.814-3.416,7.581c3.771-1.815,8.023-4.485,10.666-7.845"
        + "c2.303-2.936,4.061-5.814,4.194-9.437c0.068-1.028,0.021-2.396-0.046-3.626C91.072,71.384,90.038,73.052,88.887,74.514z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    yadj -= 27 * settings.score_scale;
    note.yBareNote = yadj;
}

function drawSixteenthNoteUp(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    xadj += 7 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M86.658,87.375c0,3.17-1.871,6.125-4.516,8.182c-2.576,2.063-5.962,3.131-8.934,2.824"
        + "c-1.893-0.189-3.645-1.037-4.854-2.223c-1.278-1.195-2.019-2.79-2.019-4.545c0-2.765,1.88-5.311,4.442-7.196"
        + "c2.562-1.944,5.808-3.213,8.519-3.149c2.37,0.058,4.309,0.335,5.528,1.537c0-18.115,0-36.23,0-54.345"
        + "c0.61-0.001,1.221,0.001,1.832,0.009c0.136,2.546,0.501,6.095,3.416,8.92c3.32,3.128,6.36,6.308,8.453,9.115"
        + "c2.768,3.687,4.715,8.121,4.715,12.518c0,2.237-0.336,4.539-0.94,6.723c1.142,2.309,1.746,4.717,1.746,7.487"
        + "c0,4.722-1.358,8.721-3.314,12.177c-0.363,0.031-0.728,0.059-1.092,0.088c1.415-2.322,3.063-7.594,2.928-10.84"
        + "c-0.135-3.605-0.401-5.686-2.692-8.548c-2.626-3.26-9.419-6.853-13.219-9.616C86.658,66.787,86.658,77.08,86.658,87.375z"
        + "M101.494,59.67c-0.136-3.618-1.887-6.481-4.181-9.488c-2.632-3.432-6.858-6.469-10.655-8.75c0.136,5.077,3.28,10.308,6.19,13.09"
        + "c3.316,3.066,4.938,4.69,6.893,7.185c0.606,0.76,1.068,1.646,1.538,2.398C101.417,62.669,101.56,60.833,101.494,59.67z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawSixteenthNoteDown(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 3 * settings.score_scale;
    yadj += 26 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M76.443,72.053c3.77-2.502,10.604-6,13.239-9.3c2.301-2.883,2.567-4.981,2.703-8.601c0.135-3.265-1.593-8.584-2.407-9.519"
        + "c0.305-0.234,0.609-0.467,0.914-0.699c1.623,2.77,2.976,6.883,2.976,11.629c0,2.513-0.47,4.739-1.482,6.854"
        + "c0.472,1.367,0.741,2.796,0.741,4.771c-0.001,4.412-1.208,10.68-5.136,15.789c-2.099,2.743-4.813,5.152-8.13,7.91"
        + "c-2.908,2.482-3.316,5.82-3.454,6.567c-0.546-0.073-1.092-0.152-1.636-0.238c0-18.813,0-37.627,0-56.441"
        + "c0-3.096,1.844-6.167,4.412-8.374c2.637-2.282,5.963-3.63,8.942-3.466c1.966,0.097,3.653,0.895,4.866,2.064"
        + "c1.282,1.178,2.018,2.776,2.018,4.529c0,2.766-1.878,5.307-4.446,7.241c-2.502,1.992-5.759,3.345-8.536,3.408"
        + "c-2.372,0.053-4.371-0.098-5.584-1.167C76.443,54.025,76.443,63.039,76.443,72.053z M89.549,67.158"
        + "c-1.964,2.49-3.591,4.117-6.912,7.063c-2.915,2.69-6.061,7.506-6.193,12.366c6.069-2.954,10.676-5.971,13.714-12.765"
        + "c0.947-2.156,1.555-5.248,1.62-7.027c0.068-1.027,0.097-2.313,0.028-3.203C91.199,64.824,90.428,66.05,89.549,67.158z";

    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawEightNoteUp(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    xadj += 7 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M86.859,28.506c0.475,3.535,0.541,7.995,4.064,12.523c2.71,3.457,5.751,6.599,7.842,9.378"
        + "c2.766,3.651,4.711,8.036,4.712,12.417c0.001,6.987-3.347,14.184-5.645,18.28c-0.364,0.02-0.729,0.039-1.094,0.057"
        + "c1.623-3.846,4.858-10.123,4.588-16.571c-0.135-3.611-1.481-7.521-3.777-10.5c-2.563-3.464-6.894-6.426-10.69-6.729"
        + "c0,13.331,0,26.661,0,39.991c0,3.167-1.832,6.122-4.475,8.185c-2.576,2.07-5.964,3.145-8.936,2.841"
        + "c-1.892-0.187-3.644-1.032-4.854-2.218c-1.279-1.196-2.019-2.789-2.019-4.544c0-2.765,1.88-5.312,4.442-7.198"
        + "c2.562-1.945,5.808-3.217,8.52-3.156c2.371,0.057,4.451,0.332,5.673,1.528c0-18.099,0-36.197,0-54.295"
        + "C85.761,28.494,86.311,28.499,86.859,28.506z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawBareNote(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 16 * settings.score_scale;    y -= 27 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 3 * settings.score_scale;
    yadj += 26 * settings.score_scale;
    //x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M67.138,95.154c2.149,3.723,8.157,4.279,13.421,1.24"
        + "c5.263-3.038,7.785-8.52,5.634-12.244c-2.147-3.722-8.157-4.278-13.419-1.239C67.512,85.949,64.988,91.431,67.138,95.154z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    note.xBareNote = xadj;
    note.yBareNote = yadj;
}

function getXBeamVertical(note, length, up, scale_factor) {
    // we want the x that is returned by drawBeamVertical
    var x;
    var y;
    if (up) {
        x = 67.138 + 18.6;
        y = 95.154 - 6.9;
        length = -length;
    }
    else {
        x = 67.138 + 0;
        y = 95.154 - 3.0;

    }
    x = x * scale_factor + note.xBareNote;
    y = y * scale_factor + note.yBareNote;
    //length = length * scale_factor;
    return { x: x, y: y + length }
}


function drawBeamVertical(ctx, note, length, up, scale_factor) {
    ctx.save();
    ctx.beginPath();
//    ctx.setTransform(scale_factor, 0, 0, scale_factor, note.xBareNote, note.yBareNote);
    ctx.lineWidth = 2;
    var x;
    var y;
    if (up) {
        x = 67.138 + 18.6;
        y = 95.154 - 6.9;
        length = -length;
    }
    else {
        x = 67.138 + 0;
        y = 95.154 - 3.0;

    }
    x = x * scale_factor + note.xBareNote;
    y = y * scale_factor + note.yBareNote;
    //length = length * scale_factor;
    ctx.moveTo(x, y);
    ctx.lineTo(x, y + length);

    ctx.stroke();
    ctx.restore();
    return { x: x, y: y + length }
}

function joinBareNotes(ctx, note1, note2, up, scale_factor) {
    ctx.save();
    ctx.lineWidth = 2;
    var x, x1, x2;
    var y, y1, y2;
    if (up) {
        x = 67.138 + 18.6;
        y = 95.154 - 6.9;
    }
    else {
        x = 67.138 + 0;
        y = 95.154 - 3.0;

    }
    if (note1.xBareNote === -1) {
        note1.xBareNote = note2.xBareNote;  // vertical line
        ctx.lineWidth = 3;
    }
    else if (note2.xBareNote === -1) {
        note2.xBareNote = note1.xBareNote;  // vertical line
        ctx.lineWidth = 3;
    }
    x1 = x * scale_factor + note1.xBareNote;
    y1 = y * scale_factor + note1.yBareNote;
    x2 = x * scale_factor + note2.xBareNote;
    y2 = y * scale_factor + note2.yBareNote;

    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.stroke();
    ctx.restore();
}

function calcVerticalLengths(beamedNotes, defaultEndpoints, tan, up, defaultVerticalLength) {
    var i;
    var verticalLengths = [];
    var minVerticalLength = 1000;   // start big
    for (i = 0; i < beamedNotes.length; i++) {
        // now we can calculate the length that we want
        var yTarget = (defaultEndpoints[i].x - defaultEndpoints[0].x) * tan + defaultEndpoints[0].y;
        var verticalLength;
        if (up)
            verticalLength = defaultVerticalLength + (defaultEndpoints[i].y - yTarget);
        else
            verticalLength = defaultVerticalLength - (defaultEndpoints[i].y - yTarget);
        verticalLengths.push(verticalLength);
        if (verticalLength < minVerticalLength)
            minVerticalLength = verticalLength;
    }
    // now adjust, we don't want a length that is to short
    var minRequiredVerticalLength = defaultVerticalLength * 0.65;
    if (minVerticalLength < minRequiredVerticalLength) {
        var addition = minRequiredVerticalLength - minVerticalLength;
        for (i = 0; i < verticalLengths.length; i++) {
            verticalLengths[i] = verticalLengths[i] + addition;
        }
    }
    return verticalLengths;
}

function drawTrapezoid(ctx, x1, y1, x2, y2, height) {
    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.lineTo(x2, y2 + height);
    ctx.lineTo(x1, y1 + height);
    ctx.lineTo(x1, y1);
    ctx.stroke();
    ctx.fill();
}

function getBeamAttributes(beamedNotes) {
    var downCount = 0;
    var upCount = 0;
    var minDuration = 1000; // start big
    var maxDuration = 0;    // start small
    var countBaseBeams = 128;   // start big
    var i, note, multinote;
    for (i = 0; i < beamedNotes.length; i++) {
        note = beamedNotes[i];
        if (note.isMultinote) {
            multinote = note.getMultinote();
            note = multinote.getNote(0);
        }
        if (note.down)
            downCount++;
        else
            upCount++;
        if (note.duration / 8 < countBaseBeams / 8)
            countBaseBeams = note.duration / 8;
        if (note.duration < minDuration)
            minDuration = note.duration;
        if (note.duration > maxDuration)
            maxDuration = note.duration;
    }
    var up = upCount > downCount;
    // typically:
    // minDuration - 8
    // maxDuration - 16
    return { up: up, maxDuration: maxDuration }
}

function resetUpdown(beamedNotes, up) {
    var i, note, multinote;
    for (i = 0; i < beamedNotes.length; i++) {
        note = beamedNotes[i];
        if (note.isMultinote) {
            console.assert(false, "should never be a multinote");
            multinote = note.getMultinote();
            note = multinote.getNote(0);
        }
        note.down = !up;
        note.directionFixed = true;
    }
}

function getHighestNotes(beamedNotes) {
    var i, note;
    var hasMultinotes = false;
    var selectedNotes = [];
    for (var i = 0; i < beamedNotes.length; i++) {
        note = beamedNotes[i];
        if (note.isMultinote) {
            var multinote = note.getMultinote();
            note = multinote.highestNote();
        }
        selectedNotes.push(note);
    }
    return selectedNotes;
}

function getLowestNotes(beamedNotes) {
    var i, note;
    var hasMultinotes = false;
    var selectedNotes = [];
    for (var i = 0; i < beamedNotes.length; i++) {
        note = beamedNotes[i];
        if (note.isMultinote) {
            var multinote = note.getMultinote();
            note = multinote.lowestNote();
        }
        selectedNotes.push(note);
    }
    return selectedNotes;
}

function getNoteRange(notes) {
    var min = 1000;
    var max = 0;
    for (var i = 0; i < notes.length; i++) {
        var note = notes[i];
        if (note.value > max)
            max = note.value;
        if (note.value < min)
            min = note.value;
    }
    return max - min;
}

function setNotesToUp(notes) {
    for (var i = 0; i < notes.length; i++) {
        var note = notes[i];
        note.down = false;
    }
}

function setNotesToDown(notes) {
    for (var i = 0; i < notes.length; i++) {
        var note = notes[i];
        note.down = true;
    }
}

function selectNotesForBeaming(beamedNotes) {
    // we want a single note for each position
    // so if there are multinotes, then we have to select one of its notes
    var i, note;
    var hasMultinotes = false;
    for (var i = 0; i < beamedNotes.length; i++) {
        note = beamedNotes[i];
        if (note.isMultinote) {
            hasMultinotes = true;
            break;
        }
    }
    if (!hasMultinotes) {
        // easy, just send back the original array
        return beamedNotes;
    }
    // if here, we need to choose the preferred multinotes
    var lowestNotes = getLowestNotes(beamedNotes);
    var highestNotes = getHighestNotes(beamedNotes);
    var lowRange = getNoteRange(lowestNotes);
    var highRange = getNoteRange(highestNotes);
    // it should also depend on where they are vertically
    if (highRange < lowRange) {
        setNotesToUp(highestNotes);
        return highestNotes;
    }
    else {
        setNotesToDown(lowestNotes);
        return lowestNotes;
    }
//    var selectedNotes = [];
//    for (i = 0; i < beamedNotes.length; i++) {
//        note = beamedNotes[i];
//        if (note.isMultinote) {
//            var multinote = note.getMultinote();
//            note = multinote.getNote(0);
//        }
//        selectedNotes.push(note);
//    }
//    return selectedNotes;
}

function drawBeam(ctx, beamedNotes) {
    var i,
        j,
        yNextLine,
        note,
        multinote;
    var scale_factor = 0.5 * settings.score_scale;
    var defaultEndpoints = [];
    var defaultVerticalLength = 50 * scale_factor;
    var selectedNotes = selectNotesForBeaming(beamedNotes);
    var beamAttributes = getBeamAttributes(selectedNotes);
    // we need to reset the updown for each note so that slur draw correctly
    resetUpdown(selectedNotes, beamAttributes.up);
    for (i = 0; i < selectedNotes.length; i++) {
        note = selectedNotes[i];
//        if (note.isMultinote) {
//            multinote = note.getMultinote();
//            note = multinote.getNote(0);
//        }
        defaultEndpoints.push(getXBeamVertical(note, defaultVerticalLength, beamAttributes.up, scale_factor));
    }
    var tan = (defaultEndpoints[beamedNotes.length - 1].y - defaultEndpoints[0].y) / (defaultEndpoints[beamedNotes.length - 1].x - defaultEndpoints[0].x);
    var verticalLengths = calcVerticalLengths(beamedNotes, defaultEndpoints, tan, beamAttributes.up, defaultVerticalLength);

    var endPoints = [];
    for (i = 0; i < selectedNotes.length; i++) {
        note = selectedNotes[i];
//        if (note.isMultinote) {
//            multinote = note.getMultinote();
//            note = multinote.getNote(0);
//        }
        var endPoint = drawBeamVertical(ctx, note, verticalLengths[i], beamAttributes.up, scale_factor);
        endPoint.duration = note.duration;    // so we can modify it
        endPoints.push(endPoint);
    }
    ctx.save();
    ctx.beginPath();
    var heightBeam = -6.5 * scale_factor;
    ctx.lineWidth = 1 * scale_factor;
    // slope must be the same
    // why don't we just use a transform skew to get the right slope?
    var yIncrement = -12 * scale_factor;
    if (beamAttributes.up) {
        yIncrement = -yIncrement;
        heightBeam = -heightBeam;
    }
    yNextLine = 0;
    for (i = 8; i <= beamAttributes.maxDuration; i *= 2) {
        for (j = 0; j < endPoints.length - 1; j++) {
            if (endPoints[j].duration >= i && endPoints[j + 1].duration >= i) {
                drawTrapezoid(ctx, endPoints[j].x, endPoints[j].y + yNextLine, endPoints[j + 1].x, endPoints[j + 1].y + yNextLine, heightBeam);
                if (endPoints[j].duration === i)
                    endPoints[j].duration = 0;  // this one is done
                if (endPoints[j + 1].duration === i && j === endPoints.length - 2)
                    endPoints[j + 1].duration = 0;
                endPoints[j].durationDone = i;
                endPoints[j + 1].durationDone = i;
            }
        }
        yNextLine += yIncrement;
    }
    // now for the short lines
    // all to the left except for the first note
    yNextLine = 0;
    var F = -tan / 3;
    for (i = 8; i <= beamAttributes.maxDuration; i *= 2) {
        for (j = 0; j < endPoints.length; j++) {
            if (endPoints[j].duration >= i && i > endPoints[j].durationDone) {
                if (j === 0) {
                    // how to see one of these??
                    // to right
                    var xRight = (3 * endPoints[j].x + endPoints[j + 1].x) / 4;
                    var h = -(endPoints[j + 1].x - endPoints[j].x) * F;
                    //ctx.moveTo(endPoints[i].x, endPoints[i].y + yNextLine);
                    //ctx.lineTo(xRight, endPoints[i].y + yNextLine + h);
                    drawTrapezoid(ctx, endPoints[j].x, endPoints[j].y + yNextLine, xRight, endPoints[j].y + yNextLine + h, heightBeam);
                }
                else {
                    // to left
                    var xLeft = (3 * endPoints[j].x + endPoints[j - 1].x) / 4;
                    var h = (endPoints[j].x - endPoints[j - 1].x) * F;
                    //ctx.moveTo(endPoints[i].x, endPoints[i].y + yNextLine);
                    //ctx.lineTo(xLeft, endPoints[i].y + yNextLine + h);
                    //ctx.lineTo(xLeft, endPoints[i].y + yNextLine + h + heightBeam);
                    //ctx.lineTo(endPoints[i].x, endPoints[i].y + yNextLine + heightBeam);
                    drawTrapezoid(ctx, xLeft, endPoints[j].y + yNextLine + h, endPoints[j].x, endPoints[j].y + yNextLine, heightBeam);
                }
                if (endPoints[j].duration === i)
                    endPoints[j].duration = 0;  // this one is done
            }
        }
        yNextLine += yIncrement;
    }

    // join up any multinotes
    connectMultinotes(ctx, beamedNotes, scale_factor);
    ctx.restore();
    // return the mid point of the beam and whether it is up or down
    // so that caller knows where to place tuplet number
    var xMid = (endPoints[0].x + endPoints[endPoints.length - 1].x) / 2;
    var yMid = (endPoints[0].y + endPoints[endPoints.length - 1].y) / 2;
    return { x: xMid, y: yMid, up: beamAttributes.up }
}

function connectMultinote(ctx, multinote, scale_factor) {
    var m;
    var direction;
    for (m = 0; m < multinote.noteCount; m++) {
        var mnote = multinote.getNote(m);
        if (mnote.directionFixed) {
            direction = mnote.down;
            break;
        }
    }
    for (m = 0; m < multinote.noteCount - 1; m++) {
        var mnote1 = multinote.getNote(m);
        var mnote2 = multinote.getNote(m + 1);
        //console.assert(!mnote1.down, "TODO: handle down")
        mnote2.down = direction;
        mnote1.down = direction;
        joinBareNotes(ctx, mnote1, mnote2, !direction, scale_factor);
    }
}


function connectMultinotes(ctx, beamedNotes, scale_factor) {
    // join up any multinotes
    for (var i = 0; i < beamedNotes.length; i++) {
        var note = beamedNotes[i];
        if (note.isMultinote) {
            var multinote = note.getMultinote();
            connectMultinote(ctx, multinote, scale_factor);
//            var m;
//            var direction;
//            for (m = 0; m < multinote.noteCount; m++) {
//                var mnote = multinote.getNote(m);
//                if (mnote.directionFixed) {
//                    direction = mnote.down;
//                    break;
//                }
//            }
//            for (m = 0; m < multinote.noteCount - 1; m++) {
//                var mnote1 = multinote.getNote(m);
//                var mnote2 = multinote.getNote(m + 1);
//                //console.assert(!mnote1.down, "TODO: handle down")
//                mnote2.down = direction;
//                mnote1.down = direction;
//                joinBareNotes(ctx, mnote1, mnote2, !direction, scale_factor);
//            }
        }
    }
}

function drawEightNoteDown(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 2 * settings.score_scale;
    yadj += 26 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M75.477,97.217c0-18.814,0-37.628,0-56.442c0-3.096,1.752-6.155,4.322-8.364c2.635-2.287,5.96-3.637,8.939-3.475"
        + "c1.967,0.096,3.654,0.896,4.868,2.063c1.278,1.178,2.017,2.776,2.017,4.53c0,2.765-1.878,5.306-4.447,7.24"
        + "c-2.499,1.992-5.759,3.347-8.536,3.412c-2.371,0.053-4.371-0.096-5.584-1.166c0,11.402,0,22.804,0,34.206"
        + "c3.772-0.016,8.094-2.752,10.668-6.178c2.305-2.931,3.653-6.865,3.79-10.496c0.271-6.497-3.021-12.621-4.583-15.96"
        + "c0.368-0.183,0.733-0.364,1.1-0.542c2.308,3.996,5.645,11.228,5.643,18.281c0,4.415-1.953,8.841-4.73,12.429"
        + "c-2.099,2.735-5.153,5.746-7.859,8.926c-3.518,4.148-3.497,8.406-3.971,11.772C76.566,97.382,76.02,97.304,75.477,97.217z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    yadj -= 26 * settings.score_scale;
    note.yBareNote = yadj;
}

function drawQuarterNoteDown(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 2 * settings.score_scale;
    yadj += 26 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M75.146,41.492c0-3.095,1.877-6.153,4.447-8.357c2.635-2.283,6.008-3.663,8.987-3.492c1.964,0.102,3.607,0.933,4.819,2.102"
        + "c1.28,1.181,1.931,2.696,1.931,4.45c0,2.767-1.792,5.386-4.36,7.315c-2.5,1.988-5.839,3.398-8.618,3.463"
        + "c-2.371,0.055-4.176-0.164-5.388-1.238c0,17.486,0,34.974,0,52.461c-0.606-0.079-1.213-0.166-1.818-0.262"
        + "C75.146,79.119,75.146,60.307,75.146,41.492z";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    yadj -= 26 * settings.score_scale;
    note.yBareNote = yadj;
}

function draw_rest_2(ctx, x, y, scale, xOffset, yOffset) {
    var scale_factor = 0.3 * settings.score_scale * scale;
    x += 20 * settings.score_scale;    y += 18 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += (3 + xOffset) * settings.score_scale;
    yadj += (26 + yOffset) * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    //polygon points=
    ctx.moveTo(100,0);
    ctx.lineTo(135,0);
    ctx.lineTo(135,15);
    ctx.lineTo(100,15);
    ctx.lineTo(100,0);
    ctx.fill();
    ctx.stroke();
}


function draw_rest_4(ctx, x, y, scale, xOffset, yOffset) {
    var scale_factor = 0.11 * settings.score_scale * scale;
    x += 45 * settings.score_scale;    y += 9 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += (3 + xOffset) * settings.score_scale;
    yadj += (26 + yOffset) * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M36.772,15.044L84.6,82.827L54.5,153.5"
                + "c0,0,27.211,48.986,29.969,55.516c0.919,2.178,0.986,3.154-0.587,1.664c-29.961-28.377-71.608,1.209-29.661,40.244"
                + "c0.058,0.053-7.926-0.807-26.728-24.295c-24.409-30.494,10.797-64.891,36.745-39.037c4.538,4.521,7.82,6.615,7.488,5.992"
                + "c-2.085-3.912-41.093-61.893-41.093-61.893l25.365-70L36.772,15.044z";
    ctx.fill();
    ctx.stroke();
}

function draw_rest_8(ctx, x, y, scale, xOffset, yOffset) {
    var scale_factor = 0.15 * settings.score_scale * scale;
    x += 40 * settings.score_scale;    y += 11 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += (3 + xOffset) * settings.score_scale;
    yadj += (26 + yOffset) * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M52.973,57.359c-5.028-2.585-8.469-7.825-8.469-13.867"
    + "c0-8.606,6.977-15.583,15.583-15.583c8.608,0,15.584,6.977,15.584,15.583c0,3.37-0.869,6.49-2.688,9.039c0.387,0.015-0.39,0,0,0"
    + "c12.153,0,23.293-6.985,28.389-17.157l-3.282,12.725C91.039,56.015,80.769,61,69.333,61c-5.851,0-11.396-1.305-16.362-3.639";
    ctx.fill();
    ctx.stroke();

    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    //polygon points=
    ctx.moveTo(102.612,30.878);
    ctx.lineTo(64.316,179.707);
    ctx.lineTo(69.173,180.729);
    ctx.lineTo(107.469,31.9);
    ctx.fill();
    ctx.stroke();
}

function draw_rest_16(ctx, x, y, scale, xOffset, yOffset) {
    var scale_factor = 0.15 * settings.score_scale * scale;
    x += 40 * settings.score_scale;    y += 11 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += (3 + xOffset) * settings.score_scale;
    yadj += (26 + yOffset) * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M52.973,57.359c-5.028-2.585-8.469-7.825-8.469-13.867"
                + "c0-8.606,6.977-15.583,15.583-15.583c8.608,0,15.584,6.977,15.584,15.583c0,3.37-0.869,6.49-2.688,9.039c0.387,0.015-0.39,0,0,0"
                + "c12.153,0,23.293-6.985,28.389-17.157l-3.282,12.725C91.039,56.015,80.769,61,69.333,61c-5.851,0-11.396-1.305-16.362-3.639";
    ctx.fill();
    ctx.stroke();

    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M38.832,112.404c-5.028-2.584-8.469-7.824-8.469-13.867"
                + "c0-8.606,6.977-15.583,15.583-15.583c8.608,0,15.584,6.977,15.584,15.583c0,3.37-0.869,6.49-2.688,9.039c0.387,0.015-0.39,0,0,0"
                + "c12.153,0,23.294-6.985,28.39-17.157l-3.283,12.725c-7.051,7.916-17.322,12.9-28.758,12.9c-5.851,0-11.396-1.305-16.362-3.639";
    ctx.fill();
    ctx.stroke();

    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    //polygon points=
    ctx.moveTo(102.612,30.878);
    ctx.lineTo(64.316,179.707);
    ctx.lineTo(69.173,180.729);
    ctx.lineTo(107.469,31.9);
    ctx.fill();
    ctx.stroke();
}


function drawQuarterNoteUp(ctx, x, y, note) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    xadj += 3 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M94.977,85.825c0,3.029-1.903,6.089-4.54,8.368c-2.567,2.276-5.963,3.625-8.943,3.457"
        + "c-1.895-0.095-3.653-0.896-4.864-2.065c-1.28-1.18-2.02-2.777-2.02-4.53c0-2.766,1.879-5.309,4.449-7.239"
        + "c2.568-1.989,5.824-3.347,8.535-3.407c2.373-0.051,4.352,0.104,5.563,1.176c0-17.488,0-34.974,0-52.462"
        + "c0.606,0.08,1.214,0.166,1.819,0.263C94.977,48.198,94.977,67.011,94.977,85.825z";
    ctx.fill();
    //ctx.text(note.position, x, top + 20)
    ctx.stroke();
    ctx.resetTransform();
    // multinotes need a join point
    note.xBareNote = -1;    // vertical line
    note.yBareNote = yadj;
}

function drawNeutral(ctx, x, y) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 13 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M79.742,43.398c0.752-0.124,1.86-0.676,1.86,0.238c0,1.367,0,2.735,0,4.103c0,2.514,0,5.026,0,7.538"
        + "c2.797-0.848,5.623-1.581,8.406-2.083c0,6.414,0,12.828,0,19.241c0,3.361,0,6.725,0,10.086c0,0.997,0.267,1.133-0.803,1.282"
        + "c-0.449,0.062-1.058,0.377-1.058-0.157c0-0.707,0-1.415,0-2.124c0-3.187,0-6.373,0-9.56c-2.798,0.848-5.624,1.579-8.406,2.076"
        + "C79.742,63.825,79.742,53.612,79.742,43.398 M81.602,68.183c0.674-0.125,6.55-1.41,6.545-1.743c0-2.466,0-4.934,0-7.4"
        + "c-0.676,0.132-6.551,1.428-6.545,1.761C81.602,63.261,81.602,65.724,81.602,68.183";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
}

function drawSharp(ctx, x, y) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 42 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 13 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M72.483,58.766v-0.258l2.716-0.727l-0.077-5.523"
            + "l-2.637,0.644V41.67h-1.869v11.687l-3.743,0.912V42.883h-1.868l0,11.84l-2.525,0.616v5.847c0,0,2.524-0.676,2.524-0.675v8.287"
            + "l-2.524,0.705v6.066c0,0,2.524-0.708,2.524-0.707v11.021h1.868V74.336l3.743-1.05c0,0.001,0,10.623,0,10.623h1.869V72.762"
            + "l2.637-0.739l0.077-6.071l-2.714,0.757L72.483,58.766z M70.616,67.23c0,0-3.743,1.045-3.743,1.045V60.01"
            + "c0,0,3.742-1.001,3.743-1.001V67.23z";



/*            "M86.616,73.463c-1.184,0.326-2.37,0.631-3.553,0.91c-0.437,0.103-0.19,1.666-0.19,2.063c0,3.025,0,6.053,0,9.077"
            + "c0,0.082,0.051,0.501-0.044,0.496c-0.282-0.005-1.665,0.085-1.824-0.126c-0.308-0.39,0-2.363,0-2.825c0-2.689,0-5.381,0-8.07"
            + "c-0.882,0.169-1.762,0.322-2.638,0.455c0-1.537,0-3.075,0-4.612c0-0.246-0.156-1.259,0.113-1.328"
            + "c0.8-0.206,1.605-0.421,2.411-0.644c0.38-0.106,0.114-3.506,0.114-3.944c0-1.485,0-2.971,0-4.456"
            + "c-0.882,0.223-1.762,0.446-2.638,0.67c0-1.548,0-3.097,0-4.646c0-0.227-0.144-1.067,0.113-1.143c0.8-0.238,1.605-0.47,2.411-0.693"
            + "c0.323-0.09,0.114-2.64,0.114-2.98c0-2.01,0-4.019,0-6.026c0-0.599-0.366-2.682,0.34-2.74c0.204-0.022,1.527-0.196,1.527-0.016"
            + "c0,0.527,0,1.057,0,1.584c0,3.206,0,6.412,0,9.617c1.184-0.325,2.37-0.63,3.553-0.909c0.438-0.101,0.19-1.668,0.19-2.063"
            + "c0-3.063,0-6.128,0-9.192c0.001-0.921,1.87-0.031,1.869-0.25c0,1.007,0,2.015,0,3.02c0,2.678,0,5.353,0,8.027"
            + "c0.881-0.17,1.761-0.323,2.637-0.459c0,1.508,0,3.014,0,4.522c0,0.421,0.2,1.028-0.243,1.141c-0.785,0.202-1.574,0.413-2.364,0.633"
            + "c-0.222,0.061-0.028,3.599-0.029,3.921c0,1.485,0,2.972,0,4.458c0.881-0.274,1.761-0.551,2.637-0.824c0,1.46,0,2.92,0,4.381"
            + "c0,0.307,0.173,1.259,0,1.533c-0.196,0.318-1.667,0.586-2.024,0.703c-0.441,0.152-0.611,0.07-0.612,0.514c0,0.608,0,1.217,0,1.824"
            + "c0,1.981,0,3.962,0,5.943c0,0.366,0.192,2.81-0.163,2.838c-0.182,0.022-1.705,0.15-1.706,0.064c0-0.435,0-0.871,0-1.306"
            + "C86.616,79.557,86.616,76.511,86.616,73.463 M82.873,68.381c1.083-0.267,2.168-0.542,3.252-0.825"
            + "c0.781-0.204,0.491-0.758,0.491-1.548c0-2.332,0-4.667,0-7c-1.083,0.265-2.168,0.542-3.251,0.825"
            + "c-0.781,0.205-0.492,0.758-0.492,1.549C82.873,63.715,82.873,66.048,82.873,68.381";*/
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
}

function drawFlat(ctx, x, y) {
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 9 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M78.988,44.466c0.336-0.064,1.859-0.465,1.86-0.095c0,0.618,0,1.236,0,1.854c0,2.597,0,5.192,0,7.788"
        + "c0,4.349,0,8.698,0,13.047c1.966-1.501,4.64-3.016,7.072-1.609c2.176,1.258,3.011,3.712,2.176,6.087"
        + "c-0.814,2.311-2.738,4.161-4.566,5.683c-2.35,1.957-4.5,3.949-6.542,5.818c0-12.68,0-25.358,0-38.036"
        + "C78.988,44.823,78.988,44.645,78.988,44.466 M80.848,79.348c2.064-2.274,5.086-5.169,5.336-8.642"
        + "c0.109-1.509-0.598-3.28-2.236-3.526c-1.215-0.216-2.61,1.201-3.021,2.222c-0.166,0.414-0.078,0.97-0.078,1.406"
        + "c0,1.109,0,2.218,0,3.327C80.848,75.872,80.848,77.608,80.848,79.348";
    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
}

function drawTrebleClef(ctx, x, y) {
    var xreturn = x;
    var scale_factor = 0.5 * settings.score_scale;
    x += 12 * settings.score_scale;    y -= 1 * settings.score_scale;
    var xadj = x - 50 * settings.score_scale;
    var yadj = y - 49 * settings.score_scale;
    yadj += 40 * settings.score_scale;
    xadj += 10 * settings.score_scale;
    x /= scale_factor; y /= scale_factor;
    ctx.beginPath();
    ctx.setTransform(scale_factor, 0, 0, scale_factor, xadj, yadj);
    ctx.path = "M90.78,59.899c6.249-0.554,12.196,1.929,15.953,6.995c3.612,4.871,4.819,11.306,3.696,17.221"
            + "c-1.16,6.112-5.056,11.215-10.215,14.57c-1.295,0.842-2.659,1.577-4.066,2.213c0.285,2.239,0.568,4.479,0.854,6.718"
            + "c0.528,4.152,1.493,8.49,1.466,12.679c-0.035,5.692-1.895,11.404-6.922,14.574c-4.966,3.131-11.522,3.213-16.473,0.059"
            + "c-5.063-3.225-8.647-9.761-6.17-15.689c2.471-5.908,10.096-7.787,14.114-2.265c2.261,3.105,2.512,7.848-0.21,10.77"
            + "c-1.623,1.741-4.773,3.128-7.147,2.193c0.065,3.102,4.807,4.653,7.333,4.56c3.651-0.158,7.243-1.458,9.604-4.348"
            + "c2.463-3.013,3.17-6.967,3.072-10.758c-0.074-2.892-0.646-5.776-1.053-8.636c-0.413-2.917-0.826-5.833-1.238-8.75"
            + "c-13.134,3.177-25.259-4.743-30.495-16.646c-2.559-5.812-3.601-12.502-2.517-18.785c1.101-6.383,4.383-12.121,8.371-17.136"
            + "c4.075-5.124,8.888-9.597,13.69-14.021c0.552-0.508,1.104-1.016,1.654-1.525c0.451-0.415,0.567-0.357,0.458-1.014"
            + "c-0.083-0.51-0.166-1.02-0.243-1.53c-0.332-2.184-0.596-4.378-0.771-6.579c-0.309-3.859-0.404-7.794-0.026-11.652"
            + "c0.504-5.149,1.931-10.345,4.083-15.045c1.073-2.347,2.381-4.647,4.132-6.56c1.234-1.35,3.232-3.132,5.184-3.223"
            + "c1.243-0.058,2.178,1.656,2.682,2.551c1.141,2.025,1.926,4.258,2.604,6.472c1.586,5.18,2.723,10.762,2.594,16.201"
            + "c-0.16,6.744-2.422,13.496-5.767,19.307c-1.759,3.054-3.859,5.908-6.23,8.514c-1.143,1.257-2.349,2.456-3.61,3.593"
            + "c-0.609,0.55-0.078,2.108,0.027,2.908c0.161,1.225,0.321,2.448,0.482,3.672C90.046,54.305,90.414,57.102,90.78,59.899"
            + " M89.357,70.108c-6.097,1.471-11.016,8.388-9.047,14.643c0.432,1.372,1.204,2.612,2.179,3.664c0.495,0.534,1.041,1.021,1.617,1.467"
            + "c0.653,0.502,1.531,0.854,2.123,1.411c0.476,0.448,0.291,1.185-0.284,1.438c-0.682,0.301-1.455-0.273-2.032-0.593"
            + "c-1.426-0.789-2.723-1.802-3.845-2.981c-4.498-4.73-5.974-11.945-3.427-17.992c1.303-3.092,3.468-5.831,6.11-7.891"
            + "c1.219-0.949,2.553-1.76,3.98-2.36c0.343-0.143,0.69-0.275,1.043-0.393c0.59-0.197,0.284-0.625,0.215-1.167"
            + "c-0.158-1.232-0.314-2.464-0.473-3.697c-0.359-2.815-0.718-5.633-1.077-8.449c-9.868,7.855-21.147,17.782-20.896,31.557"
            + "c0.227,12.347,11.517,22.026,23.729,21.048c1.117-0.09,2.225-0.277,3.313-0.55c0.794-0.2,0.36-1.004,0.271-1.705"
            + "c-0.09-0.702-0.18-1.404-0.27-2.107c-0.359-2.812-0.717-5.624-1.076-8.437C90.793,81.378,90.074,75.744,89.357,70.108"
            + " M101.734,7.26c0.114-3.351-2.348-7.081-6.034-5.687c-2.881,1.089-4.753,3.944-6.029,6.601c-3.495,7.271-3.876,15.193-2.535,23.05"
            + "C94.494,25.532,101.441,16.967,101.734,7.26 M95.807,98.385c6.131-3.029,10.09-10,8.939-16.826"
            + "c-1.072-6.361-6.154-11.264-12.673-11.62C93.314,79.421,94.559,88.903,95.807,98.385";

    ctx.fill();
    ctx.stroke();
    ctx.resetTransform();
    return xreturn + 35 * settings.score_scale;
}

function measureTrebleClef(x) {
    return x + 35 * settings.score_scale;
}

function collectNeutrals(prevFlats, prevSharps) {
    var neutralPositions = [];
    switch (prevSharps)
    {
        case 7:
            neutralPositions.push(-4);  // B
            // fall through
        case 6:
            neutralPositions.push(-1);  // E
            // fall through
        case 5:
            neutralPositions.push(-5);  // A
            // fall through
        case 4:
            neutralPositions.push(-2);  // D
            // fall through
        case 3:
            neutralPositions.push(1);  // G
            // fall through
        case 2:
            neutralPositions.push(-3);  // C
            // fall through
        case 1:
            neutralPositions.push(0);  // F
            // fall through
    }
    switch (prevFlats)
    {
        case 7:
            neutralPositions.push(-7);  // F
            // fall through
        case 6:
            neutralPositions.push(-3);  // C
            // fall through
        case 5:
            neutralPositions.push(1);  // G
            // fall through
        case 4:
            neutralPositions.push(-2);  // D
            // fall through
        case 3:
            neutralPositions.push(-5);  // A
            // fall through
        case 2:
            neutralPositions.push(-1);  // E
            // fall through
        case 1:
            neutralPositions.push(-4);  // B
            // fall through
    }
    return neutralPositions;
}

function collectSharps(sharps) {
    var sharpPositions = [];
    switch (sharps)
    {
        case 7:
            sharpPositions.push(-4);  // B
            // fall through
        case 6:
            sharpPositions.push(-1);  // E
            // fall through
        case 5:
            sharpPositions.push(-5);  // A
            // fall through
        case 4:
            sharpPositions.push(-2);  // D
            // fall through
        case 3:
            sharpPositions.push(1);  // G
            // fall through
        case 2:
            sharpPositions.push(-3);  // C
            // fall through
        case 1:
            sharpPositions.push(0);  // F
            // fall through
    }
    return sharpPositions;
}

function collectFlats(flats) {
    var flatPositions = [];
    switch (flats)
    {
        case 7:
            sharpPositions.push(-7);  // F
            // fall through
        case 6:
            flatPositions.push(-3);  // C
            // fall through
        case 5:
            flatPositions.push(1);  // G
            // fall through
        case 4:
            flatPositions.push(-2);  // D
            // fall through
        case 3:
            flatPositions.push(-5);  // A
            // fall through
        case 2:
            flatPositions.push(-1);  // E
            // fall through
        case 1:
            flatPositions.push(-4);  // B
            // fall through
    }
    return flatPositions;
}

function drawKey(ctx, x, top, flats, sharps, prevFlats, prevSharps) {
    var y,
        i,
        xreturn = x,
        sharpPositions = collectSharps(sharps),
        flatPositions = collectFlats(flats),
        neutralPositions = collectNeutrals(prevFlats, prevSharps),
        count = 0;
    if (sharpPositions.length === 0 && flatPositions.length === 0) {
        for (i = neutralPositions.length - 1; i >= 0; i--) {
            if (sharpPositions.indexOf(neutralPositions[i]) === -1 && flatPositions.indexOf(neutralPositions[i]) === -1) {
                y = top - score.spacing * neutralPositions[i] / 2;
                y += score.spacing / 2;
                drawNeutral(ctx, x, y);
                x += score.spacing / 1.5;
                count++;
            }
        }
    }
    for (i = sharpPositions.length - 1; i >= 0; i--) {
        y = top - score.spacing * sharpPositions[i] / 2;
        y += score.spacing / 2;
        drawSharp(ctx, x, y);
        x += score.spacing / 1.5;
        count++;
    }
    for (i = flatPositions.length - 1; i >= 0; i--) {
        y = top - score.spacing * flatPositions[i] / 2;
        y += score.spacing / 2;
        drawFlat(ctx, x, y);
        x += score.spacing / 1.5;
        count++;
    }
    if (count > 0)
        xreturn += score.note_spacing / 1.5 + (count - 1) * score.note_spacing / 2;

    return xreturn;
}

function measureKey(x, flats, sharps, prevFlats, prevSharps) {
    var i,
    sharpPositions = collectSharps(sharps),
    flatPositions = collectFlats(flats),
    neutralPositions = collectNeutrals(prevFlats, prevSharps),
    count = 0;
    if (sharpPositions.length === 0 && flatPositions.length === 0) {
        for (i = neutralPositions.length - 1; i >= 0; i--) {
            if (sharpPositions.indexOf(neutralPositions[i]) === -1 && flatPositions.indexOf(neutralPositions[i]) === -1) {
                count++;
            }
        }
    }
    for (i = sharpPositions.length - 1; i >= 0; i--) {
        count++;
    }
    for (i = flatPositions.length - 1; i >= 0; i--) {
        count++;
    }
    if (count > 0)
        x += score.note_spacing / 1.5 + (count - 1) * score.note_spacing / 2;
    return x;
}

function drawTie(ctx, note1, note2, tab)
{
    var yPos1 = note1.yPos;
    var yPos2 = note2.yPos;
    if (tab) {
        yPos1 = note1.yPosTab;
        yPos2 = note2.yPosTab;
    }
    var x = (note2.xPos - note1.xPos);
    var y = (yPos2 - yPos1);
    var W2 = x * x + y * y;
    var H = 5 * settings.score_scale;
    var radius = H / 2 + W2 / (8 * H);
    var alpha;
    var xC;
    var yC;
    var phi1;
    var phi2;
    if (!note1.down) {
        alpha = Math.atan(y / x);
        xC = -(radius - H) * Math.sin(alpha) + (note2.xPos + note1.xPos) / 2;
        yC = -(radius - H) * Math.cos(alpha) + (yPos2 + yPos1) / 2;
        phi1 = Math.acos((note2.xPos - xC) / radius);
        phi2 = Math.acos((note1.xPos - xC) / radius);
/*        if (y < 0) {
            phi1 -= 3 * Math.PI / 4;
            phi2 -= 3 * Math.PI / 4;
        }*/
    }
    else {
        alpha = Math.atan(-y / x);
        xC = (radius - H) * Math.sin(alpha) + (note2.xPos + note1.xPos) / 2;
        yC = (radius - H) * Math.cos(alpha) + (yPos2 + yPos1) / 2;
        phi1 = Math.acos((note2.xPos - xC) / radius) + Math.PI;
        phi2 = Math.acos((note1.xPos - xC) / radius) + Math.PI;
        yC -= score.spacing;
/*        if (y < 0) {
            phi1 -= 1 * Math.PI / 4;
            phi2 -= 3 * Math.PI / 4;
        }*/
    }
/*   phi1 = 0;
    phi2 = 2*Math.PI;
*/
/*    console.log("note1: ", note1.down ? "down" : "up");
    console.log("x: ", x);
    console.log("y: ", y);
    console.log("H: ", H);
    console.log("r: ", radius);
    console.log("W2: ", W2);
    console.log("xC: ", xC);
    console.log("yC: ", yC);
*/
    ctx.save();
    ctx.lineWidth = 1;
    ctx.beginPath();
    ctx.arc(xC, yC, radius, phi1, phi2);

//    ctx.moveTo(note1.xPos, note1.yPos);
//    ctx.lineTo(note2.xPos, note2.yPos);
    ctx.stroke();
    ctx.restore();
}

function drawNote(note, ctx, x, top, bottom, beamed) {
    var y = top - score.spacing * note.position / 2;
    var notePos = y;    // for mini-lines
    y += score.spacing / 2;
    if (!note.isRest) {
        var topLine = top;
        var bottomLine = bottom;
        while (topLine - score.spacing >= notePos)
        {
            // above the stave, put in some lines
            topLine -= score.spacing;
            ctx.beginPath();
            ctx.moveTo(x - 1 * settings.score_scale, topLine);
            ctx.lineTo(x + 16 * settings.score_scale, topLine);
            ctx.stroke();
        }
        while (bottomLine + score.spacing <= notePos)
        {
            // below the stave, put in some lines
            bottomLine += score.spacing;
            ctx.beginPath();
            ctx.moveTo(x - 5 * settings.score_scale, bottomLine);
            ctx.lineTo(x + 12 * settings.score_scale, bottomLine);
            ctx.stroke();
        }
    }

    //note.configureDuration();
    var duration = note.duration;
    if (note.isFlat)
    {
        drawFlat(ctx, x, y);
        x += score.note_spacing / 2;
    }
    else if (note.isNeutral)
    {
        drawNeutral(ctx, x, y);
        x += score.note_spacing / 2;
    }
    else if (note.isSharp)
    {
        drawSharp(ctx, x, y);
        x += score.note_spacing / 2;
    }
    var xreturn = x + score.note_spacing;
    if (duration <= 2)
        xreturn += score.note_spacing

    // -4 is mid position, above is -3 and below is -5
    // so -4 or more and we do down
    var down = false;
    if (note.position >= -4)
        down = true;
    note.xPos = x;
    note.yPos = y;
    note.top = top;
    note.bottom = bottom;
    if (!note.directionFixed)
        note.down = down;
    else
        down = note.down;
    if (note.isRest)
    {
        // put it in the middle
        y = top - score.spacing * - 4 / 2;
        var t = "";
        if (note.hasPrefix)
            t += "(" + note.prefix + ")";
        ctx.save();
        if (duration === 16)
            draw_rest_16(ctx, x, y, 1, 0, 0);
        else if (duration === 8)
            draw_rest_8(ctx, x, y, 1, 0, 0);
        else if (duration === 4)
            draw_rest_4(ctx, x, y, 1, 0, 0);
        else if (duration === 2)
            draw_rest_2(ctx, x, y, 1, 0, 0);
        else
            t += "R";
        ctx.restore();
        if (note.dotted)
            t += ".";
        if (note.doubleDotted)
            t += "..";
        if (note.hasPostfix)
            t += "(" + note.postfix + ")";
        ctx.font = score.fontChord; //"12px sans-seri";
        ctx.text(t, x, y);
        ctx.stroke();
        ctx.fill();
    }
    else if (duration === 1 || duration === 2 || duration === 4 || duration === 8 || duration === 16 || duration === 32)
    {
        switch (duration) {
            case 1:
                drawWholeNote(ctx, x, y);
                break;
            case 2:
                if (beamed) // it's never beamed but might be multinoted
                    drawBareHalfNote(ctx, x, y, note);
                else if (down)
                    drawHalfNoteDown(ctx, x, y, note);  // adjusted
                else
                    drawHalfNoteUp(ctx, x, y, note);    // OK
                break;
            case 4:
                if (beamed)
                    drawBareNote(ctx, x, y, note);
                else if (down)
                    drawQuarterNoteDown(ctx, x, y, note);   // adjusted
                else
                    drawQuarterNoteUp(ctx, x, y, note); // adjusted
                break;
            case 8:
                if (beamed)
                    drawBareNote(ctx, x, y, note);
                else if (down)
                    drawEightNoteDown(ctx, x, y, note);
                else
                    drawEightNoteUp(ctx, x, y, note);   // adjusted
                break;
            case 16:
                if (beamed)
                    drawBareNote(ctx, x, y, note);
                else if (down)
                    drawSixteenthNoteDown(ctx, x, y, note); // OK
                else
                    drawSixteenthNoteUp(ctx, x, y, note);   // adjusted
                break;
            case 32:
                if (beamed)
                    drawBareNote(ctx, x, y, note);
                else if (down)
                    drawThirtySecondNoteDown(ctx, x, y, note);  // adjusted
                else
                    drawThirtySecondNoteUp(ctx, x, y, note);    // adjusted
                break;
        }
        if (note.hatted) {
            drawHat(ctx, x, top);
        }
        if (note.dotAbove) {
            drawDotAbove(ctx, x, y);
        }
        if (note.dotted) {
            var t = ".";
            ctx.font = score.fontChord; // "12px sans-seri";
            ctx.text(t, x + score.note_spacing / 2, y);
            ctx.stroke();
            ctx.fill();
            xreturn += score.note_spacing / 2;
        }
        if (note.doubleDotted) {
            var t = "..";
            ctx.font = score.fontChord; // "12px sans-seri";
            ctx.text(t, x + score.note_spacing / 2, y);
            ctx.stroke();
            ctx.fill();
            xreturn += score.note_spacing / 2;
        }
    }
    else
    {
        var t = "";
        if (note.hasPrefix)
            t += "(" + note.prefix + ")";
        t += duration.toString();
        if (note.dotted)
            t += ".";
        if (note.doubleDotted)
            t += "..";
        if (note.hasPostfix)
            t += "(" + note.postfix + ")";
        ctx.font = score.fontChord; // "12px sans-seri";
        ctx.text(t, x, y);
        ctx.stroke();
    }

    return xreturn;
}

function measureNote(note, x) {
    //note.configureDuration();   // to get dotted set
    var duration = note.duration;
    if (duration <= 2)
        x += score.note_spacing;
    x += score.note_spacing;
    //if (note.isSharp || note.isNeutral || note.isFlat)
    if (note.isIncidental)
        x += score.note_spacing / 2;
    if (note.dotted)
        x += score.note_spacing / 2;
    if (note.doubleDotted)
        x += score.note_spacing / 2;
    return x;
}

function measureScoreLength(bars, flats, sharps, prevFlats, prevSharps, prev)
{
    var xPos = 0;
    xPos = measureTrebleClef(xPos);
    xPos = measureKey(xPos, flats, sharps, prevFlats, prevSharps);
//    xPos = measureMeter(xPos, meter);
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var bar = bars[i];
        var barFlats = bar.flats();
        var barSharps = bar.sharps();
        if (barFlats !== flats || barSharps !== sharps) {
            xPos = measureKey(xPos, barFlats, barSharps, flats, sharps);
            flats = barFlats;
            sharps = barSharps;
        }
        if (prev.Meter !== bar.meter) {
            prev.Meter = bar.meter;
            xPos = measureMeter(xPos, prev.Meter);
        }

        xPos = measureStartBar(bar.start, xPos);
        var noteCount = bar.notes.length;
        if (noteCount > 0) {
            for (var n = 0; n < noteCount; n++) {
                var note = bar.notes[n];
                if (note.isTuplet)
                {
                    var tuplet = note.getTuplet();
                    for (var t = 0; t < tuplet.noteCount; t++)
                    {
                        var tnote = tuplet.getNote(t);
                        xPos = measureNote(tnote, xPos);
                    }
                }
                else if (note.isMultinote) {
                    var multinote = note.getMultinote();
                    for (var m = 0; m < multinote.noteCount; t++)
                    {
                        var mnote = multinote.getNote(m);
                        xPos = measureNote(mnote, xPos);
                        break;
                        // if we ever need to handle different legnth multinotes, then find max instead of breaking on first note
                    }
                }
                else
                    xPos = measureNote(note, xPos);
            }
        }
        xPos = measureEndBar(bar.end, xPos);
    }
    // we'll get an extra increment, for now assume it is a bar
    // if it's a note it will be bigger
    xPos -= score.double_bar_spacing;
    // and if it's a note, we don't mind the extra length since there is not vertical bar to match with
    return xPos;
}

function calcNumberOnTab(note, openStringValues) {
    var value = note.value,
        i,
        fret = -1,
        stringNumber = -1;
    if (!note.isRest)
    {
        // openStringValues starts with bass string but we shouldn't assume that it alway is
        var fretPossibilities = [];
        for (i = 0; i < openStringValues.length; i++) {
            fret = value - openStringValues[i];
            fretPossibilities.push(fret);
        }
        // now we want the lowest >= 0 fret
        fret = 100;  // start big
        for (i = 0; i < fretPossibilities.length; i++) {
            if (fretPossibilities[i] >= 0) {
                if (fretPossibilities[i] < fret) {
                    fret = fretPossibilities[i];
                    stringNumber = i;
                }
            }
        }
    }
    note.stringNumber = stringNumber;
    note.fret = fret;
}

function drawNumberOnTab(ctx, note, top, bottom) {
    var i,
        stringY = bottom;   // top;
    if (note.fret >= 0) {
        //stringY += score.spacing * note.stringNumber;
        stringY -= score.spacing * note.stringNumber;
        ctx.beginPath();
        ctx.text(note.fret, note.xPos, stringY);
        note.yPosTab = stringY;
        note.bottom = bottom;
        ctx.stroke();
        ctx.fill();
        //addLeg(ctx, note, bottom);
    }
}

function getFirstMultinote(note) {
    if (note.isMultinote) {
        var multinote = note.getMultinote();
        return multinote.getNote(0);
    }
    return note;
}

function drawTabBeam(ctx, beamedNotes, yPos, lineWidth) {
    var i, j, xMid, yMid, note, multinote;
    var scale_factor = 0.5 * settings.score_scale;
    var heightBeam = -4.5 * scale_factor;
    var yIncrement = 2* heightBeam; //-12 * scale_factor;
    var endPoints = [];
    var beamAttributes = getBeamAttributes(beamedNotes);
    for (i = 0; i < beamedNotes.length; i++) {
        note = getFirstMultinote(beamedNotes[i]);
        var endPoint = { x: note.xPos + score.legXOffset, y: yPos }
        endPoint.duration = note.duration;    // so we can modify it
        endPoints.push(endPoint);
    }
    xMid = (endPoints[0].x + endPoints[endPoints.length - 1].x) / 2;
    xMid -= score.legXOffset;   // this will line up characters
    yMid = yPos;
    ctx.save();
    var yNextLine = 0;
    for (i = 8; i <= beamAttributes.maxDuration; i *= 2) {
        for (j = 0; j < endPoints.length - 1; j++) {
            if (endPoints[j].duration >= i && endPoints[j + 1].duration >= i) {
                drawTrapezoid(ctx, endPoints[j].x, endPoints[j].y + yNextLine, endPoints[j + 1].x, endPoints[j + 1].y + yNextLine, heightBeam);
                if (endPoints[j].duration === i)
                    endPoints[j].duration = 0;  // this one is done
                if (endPoints[j + 1].duration === i && j === endPoints.length - 2)
                    endPoints[j + 1].duration = 0;
                endPoints[j].durationDone = i;
                endPoints[j + 1].durationDone = i;
            }
        }
        yNextLine += yIncrement;
    }
    // now for the short lines
    // all to the left except for the first note
    yNextLine = 0;
    var F = 0;  //-tan / 3;
    for (i = 8; i <= beamAttributes.maxDuration; i *= 2) {
        for (j = 0; j < endPoints.length; j++) {
            if (endPoints[j].duration >= i && i > endPoints[j].durationDone) {
                if (j === 0) {
                    // how to see one of these??
                    // to right
                    var xRight = (3 * endPoints[j].x + endPoints[j + 1].x) / 4;
                    var h = -(endPoints[j + 1].x - endPoints[j].x) * F;
                    //ctx.moveTo(endPoints[i].x, endPoints[i].y + yNextLine);
                    //ctx.lineTo(xRight, endPoints[i].y + yNextLine + h);
                    drawTrapezoid(ctx, endPoints[j].x, endPoints[j].y + yNextLine, xRight, endPoints[j].y + yNextLine + h, heightBeam);
                }
                else {
                    // to left
                    var xLeft = (3 * endPoints[j].x + endPoints[j - 1].x) / 4;
                    var h = (endPoints[j].x - endPoints[j - 1].x) * F;
                    //ctx.moveTo(endPoints[i].x, endPoints[i].y + yNextLine);
                    //ctx.lineTo(xLeft, endPoints[i].y + yNextLine + h);
                    //ctx.lineTo(xLeft, endPoints[i].y + yNextLine + h + heightBeam);
                    //ctx.lineTo(endPoints[i].x, endPoints[i].y + yNextLine + heightBeam);
                    drawTrapezoid(ctx, xLeft, endPoints[j].y + yNextLine + h, endPoints[j].x, endPoints[j].y + yNextLine, heightBeam);
                }
                if (endPoints[j].duration === i)
                    endPoints[j].duration = 0;  // this one is done
            }
        }
        yNextLine += yIncrement;
    }
    ctx.restore();
    // up was being initialised to false and then not touched
    return { x: xMid, y: yMid, up: false }
}

function addLegTick(ctx, xPos, yPos, tickLength, tickHeight) {
    var scale_factor = 0.5 * settings.score_scale;
    var angle_factor = 1;
    ctx.save();
    ctx.lineWidth = 0.5;
    ctx.beginPath();
    ctx.moveTo(xPos, yPos - tickHeight);
    ctx.lineTo(xPos, yPos);
    ctx.lineTo(xPos + tickLength, yPos - tickHeight / angle_factor);
    ctx.lineTo(xPos + tickLength, yPos - tickHeight - tickHeight / angle_factor);
    ctx.stroke();
    ctx.fill();
    ctx.restore();
}

function addLeg(ctx, note, top, bottom, defaultVerticalLength, beamed) {
    if (note.isRest)
    {
        // put it in the middle
        var y = bottom + score.spacing * 1.1;
        var x = note.xPos + score.legXOffset;
        var t = "";
        ctx.save();
        var scale = 0.6;
        if (note.duration === 16)
            draw_rest_16(ctx, x, y, scale, 0, 0);
        else if (note.duration === 8)
            draw_rest_8(ctx, x, y, scale, 0, 0);
        else if (note.duration === 4)
            draw_rest_4(ctx, x, y, scale, 0, 0);
        else if (note.duration === 2)
            draw_rest_2(ctx, x, y, scale, 10, 0);
        if (note.dotted)
            t += ".";
        if (note.doubleDotted)
            t += "..";
        ctx.font = score.fontChord; //"12px sans-seri";
        ctx.text(t, x, y);
        ctx.stroke();
        ctx.fill();
        ctx.restore();
    }
    else if (note.fret >= 0) {
        var scale_factor = 0.5 * settings.score_scale;
        var verticalLineWidth = 2 * scale_factor;
        var xPos = note.xPos + score.legXOffset;
        ctx.save();
        var yStart = bottom;    //note.yPosTab + 5 * scale_factor;
        if (note.duration === 1) {
            // small square
            ctx.lineWidth = verticalLineWidth / 3;
            ctx.beginPath();
            ctx.moveTo(xPos, yStart);
            ctx.lineTo(xPos, yStart + verticalLineWidth * 4);
            ctx.lineTo(xPos + verticalLineWidth * 4, yStart + verticalLineWidth * 4);
            ctx.lineTo(xPos + verticalLineWidth * 4, yStart);
            ctx.lineTo(xPos, yStart);
            ctx.stroke();
        }
        else if (note.duration === 2) {
            // hollow leg
            ctx.lineWidth = verticalLineWidth / 3;
            ctx.beginPath();
            ctx.moveTo(xPos, yStart);
            ctx.lineTo(xPos, yStart + defaultVerticalLength);
            ctx.lineTo(xPos + verticalLineWidth * 2, yStart + defaultVerticalLength);
            ctx.lineTo(xPos + verticalLineWidth * 2, yStart);
            ctx.lineTo(xPos, yStart);
            ctx.stroke();
        }
        else {
            // normal leg
            ctx.lineWidth = verticalLineWidth;
            ctx.beginPath();
            ctx.moveTo(xPos, yStart);
            ctx.lineTo(xPos, yStart + defaultVerticalLength);
            ctx.stroke();
        }
        var tickHeight = 4.5 * scale_factor;
        var tickLength = 10 * scale_factor;
        var tickSpacing = 10 * scale_factor;
        if (note.duration === 32) {
            if (!beamed) {
                ctx.beginPath();
                addLegTick(ctx, xPos, yStart + defaultVerticalLength - tickSpacing * 2, tickLength, tickHeight)
                addLegTick(ctx, xPos, yStart + defaultVerticalLength - tickSpacing, tickLength, tickHeight)
                addLegTick(ctx, xPos, yStart + defaultVerticalLength, tickLength, tickHeight)
                ctx.stroke();
            }
        }
        else if (note.duration === 16) {
            if (!beamed) {
                ctx.beginPath();
                addLegTick(ctx, xPos, yStart + defaultVerticalLength - tickSpacing, tickLength, tickHeight)
                addLegTick(ctx, xPos, yStart + defaultVerticalLength, tickLength, tickHeight)
                ctx.stroke();
            }
        }
        else if (note.duration === 8) {
            if (!beamed) {
                addLegTick(ctx, xPos, yStart + defaultVerticalLength, tickLength, tickHeight)
            }
        }
        else if (note.duration === 4) {
            //ctx.lineTo(xPos + 40, yStart + defaultVerticalLength);
        }
        else if (note.duration === 2) {
            //ctx.lineTo(xPos + 20, yStart + defaultVerticalLength);
        }
        else if (note.duration === 1) {
            //ctx.lineTo(xPos + 20, yStart + defaultVerticalLength);
        }
        ctx.restore();
        ctx.save();
        if (note.hatted && score.hideStaff) {
            drawHat(ctx, note.xPos, top - score.spacing / 2);
        }
        ctx.beginPath();
        var t = "";
        if (note.dotted)
            t += ".";
        if (note.doubleDotted)
            t += "..";
        ctx.font = score.fontChord; //"12px sans-seri";
        ctx.text(t, xPos + tickLength, yStart);
        ctx.stroke();
        ctx.fill();
        ctx.restore();
    }
}

function calcTabNotesForBar(bar, openStringValues) {
    var noteCount = bar.notes.length;
    if (noteCount > 0) {
        for (var n = 0; n < noteCount; n++) {
            var note = bar.notes[n];
            if (note.isTuplet) {
                var tuplet = note.getTuplet();
                for (var t = 0; t < tuplet.noteCount; t++) {
                    var tnote = tuplet.getNote(t);
                    calcNumberOnTab(tnote, openStringValues);
                }
            }
            else if (note.isMultinote) {
                var multinote = note.getMultinote();
                for (var m = 0; m < multinote.noteCount; m++) {
                    var mnote = multinote.getNote(m);
                    calcNumberOnTab(mnote, openStringValues);
                }
            }
            else {
                calcNumberOnTab(note, openStringValues);
            }
        }
    }
}

function drawTabNotesForBar(ctx, bar, top, bottom) {
    // position numbers directly under notes
    var tupletNumberDrawn = false;
    var noteCount = bar.notes.length,
            note,
            tuplet,
            multinote,
            n, t, m,
            tnote, mnote;
    if (noteCount > 0) {
        ctx.save();
        ctx.font = score.fontPart;
        for (n = 0; n < noteCount; n++) {
            note = bar.notes[n];
            if (note.isTuplet) {
                tuplet = note.getTuplet();
                for (t = 0; t < tuplet.noteCount; t++) {
                    tnote = tuplet.getNote(t);
                    drawNumberOnTab(ctx, tnote, top, bottom);
                }
            }
            else if (note.isMultinote) {
                multinote = note.getMultinote();
                for (m = 0; m < multinote.noteCount; m++) {
                    mnote = multinote.getNote(m);
                    drawNumberOnTab(ctx, mnote, top, bottom);
                }
            }
            else {
                drawNumberOnTab(ctx, note, top, bottom);
            }
        }
        if (score.defaultLegLength) {
            // now the legs
            var beamedNotes = [];
            var scale_factor = 0.5 * settings.score_scale;
            for (n = 0; n < noteCount; n++) {
                note = bar.notes[n];
                if (note.isTuplet) {
                    tuplet = note.getTuplet();
                    var beamAttributes = null;
                    for (t = 0; t < tuplet.noteCount; t++) {
                        tnote = tuplet.getNote(t);
                        if (tnote.beamWithNext || beamedNotes.length > 0) {
                            beamedNotes.push(tnote);
                        }
                        addLeg(ctx, tnote, top, bottom + score.legYOffset, score.defaultLegLength, beamedNotes.length !== 0);
                        if (beamedNotes.length && !tnote.beamWithNext) {
                            // add the beam
                            beamAttributes = drawTabBeam(ctx, beamedNotes, bottom + score.legYOffset + score.defaultLegLength, 6 * scale_factor);
                            beamedNotes.length = 0;
                        }
                    }
                    if (beamAttributes) {
                        drawTupletNumberOnBeam(ctx, beamAttributes, tuplet.noteCount);
                        tupletNumberDrawn = true;
                    }
                }
                else {
                    if (note.beamWithNext || beamedNotes.length > 0) {
                        beamedNotes.push(note);
                    }
                    if (note.isMultinote) {
                        // assume all same duration
                        multinote = note.getMultinote();
                        mnote = note.getNote(0);
                        addLeg(ctx, mnote, top, bottom + score.legYOffset, score.defaultLegLength, beamedNotes.length !== 0);
                    }
                    else
                        addLeg(ctx, note, top, bottom + score.legYOffset, score.defaultLegLength, beamedNotes.length !== 0);
                    if (beamedNotes.length && !note.beamWithNext) {
                        // add the beam
                        drawTabBeam(ctx, beamedNotes, bottom + score.legYOffset + score.defaultLegLength, 6 * scale_factor);
                        beamedNotes.length = 0;
                    }
                }
            }
        }
        ctx.restore();
    }
    return tupletNumberDrawn;
}

function findNearestTo(position, note1, note2) {
    // caller wants the note that is nearest to position
    if (Math.abs(position - note1.position) > Math.abs(position - note2.position))
        return note2;
    return note1;
}

function drawNotesForBar(ctx, bar, xPos, top, bottom, offsetChord, useBeams) {
    var chordName;
    var noteCount = bar.notes.length;
    if (noteCount > 0) {
        var beamedNotes = [];

        for (var n = 0; n < noteCount; n++) {
            var note = bar.notes[n];
            if (note.isTuplet) {
                var tuplet = note.getTuplet();
                if (tuplet.hasChord) {
                    chordName = tuplet.getChordName(fretboard.capoPosition);
                    writeChord(chordName, ctx, xPos, top - offsetChord);
                }
                var posFirst = xPos;
                var beamAttributes = null;
                for (var t = 0; t < tuplet.noteCount; t++) {
                    var tnote = tuplet.getNote(t);
                    if (tnote.hasChord) {
                        chordName = tnote.getChordName(fretboard.capoPosition);
                        writeChord(chordName, ctx, xPos, top - offsetChord);
                    }
                    if (useBeams && (tnote.beamWithNext || beamedNotes.length > 0)) {
                        beamedNotes.push(tnote);
                    }
                    xPos = drawNote(tnote, ctx, xPos, top, bottom, beamedNotes.length);
                    if (beamedNotes.length && !tnote.beamWithNext) {
                        // add the beam
                        beamAttributes = drawBeam(ctx, beamedNotes);
                        beamedNotes.length = 0;
                    }
                }
                if (beamAttributes) {
                    drawTupletNumberOnBeam(ctx, beamAttributes, tuplet.noteCount);
                }
                else {
                    var posLast = xPos;
                    var posMid = (posLast + posFirst) / 2;  // not quite right
                    posMid -= score.note_spacing / 2;
                    var midNote = tuplet.getNote(tuplet.noteCount / 2);
                    drawTupletNumber(midNote, ctx, posMid, top, tuplet.noteCount);
                }
            }
            else {
                var multinote;
                if (note.hasChord) {
                    chordName = note.getChordName(fretboard.capoPosition);
                    writeChord(chordName, ctx, xPos, top - offsetChord);
                    //writeChord(note.chordName, ctx, xPos, top - offsetChord);
                }
                if (useBeams && (note.beamWithNext || beamedNotes.length > 0)) {
                    beamedNotes.push(note);
                }
                if (note.isMultinote) {
                    var mnote;
                    var xPosOrig
                    multinote = note.getMultinote();
                    if (beamedNotes.length) {
                        // will draw bare notes so no issues
                        xPosOrig = xPos;
                        for (var m = 0; m < multinote.noteCount; m++) {
                            mnote = multinote.getNote(m);
                            xPos = drawNote(mnote, ctx, xPosOrig, top, bottom, beamedNotes.length);
                        }
                    }
                    else {
                        // todo: choose best
                        var highest = multinote.highestNote();
                        var lowest = multinote.lowestNote();
                        var best = findNearestTo(-4, highest, lowest);
                        if (best === highest)
                            best.down = false;
                        else
                            best.down = true;
                        best.directionFixed = true;

                        xPosOrig = xPos;
                        xPos = drawNote(best, ctx, xPosOrig, top, bottom, beamedNotes.length);
                        for (var m = 0; m < multinote.noteCount; m++) {
                            mnote = multinote.getNote(m);
                            if (mnote !== best)
                                xPos = drawNote(mnote, ctx, xPosOrig, top, bottom, true);   // draw bare notes
                        }
                        var scale_factor = 0.5 * settings.score_scale;
                        connectMultinote(ctx, multinote, scale_factor);
                    }
                }
                else
                    xPos = drawNote(note, ctx, xPos, top, bottom, beamedNotes.length);

                if (beamedNotes.length && !note.beamWithNext) {
                    // add the beam
                    drawBeam(ctx, beamedNotes);
                    beamedNotes.length = 0;
                }
            }
        }
    }
    return xPos;
}

function drawTies(ctx, bars, tab) {
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var bar = bars[i];
        var noteCount = bar.notes.length;
        for (var tie = 0; tie < noteCount; tie++) {
            var note1 = bar.notes[tie];
            var note2 = null;
            var note3 = null;
            if (note1.tied)
            {
                if (tie === noteCount - 1) {
                    // tied to the note in the next bar
                    var iNext = i + 1;
                    if (iNext < countBars) {
                        // there is a next bar on this line
                        var nextBar = bars[iNext];
                        if (nextBar.notes.length) {
                            note2 = nextBar.notes[0];
                            if (nextBar.repeatEndingNumber === 1) {
                                // we also have to tie to the first note in repeat ending 2
                                iNext = iNext + 1;
                                while (iNext < countBars) {
                                    var nextRepeatBar = bars[iNext];
                                    if (nextRepeatBar.repeatEndingNumber === 2) {
                                        // this it the bar we want
                                        if (nextRepeatBar.notes.length) {
                                            note3 = nextRepeatBar.notes[0];
                                        }
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
                else {
                    note2 = bar.notes[tie + 1];
                }
                if (note2 !== null)
                    drawTie(ctx, note1, note2, tab);
                if (note3 !== null)
                    drawTie(ctx, note1, note3, tab);
            }
        }
    }
}

function getListOfNotes(bars) {
    var notes = [];
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var bar = bars[i];
        var noteCount = bar.notes.length;
        for (var j = 0; j < noteCount; j++) {
            var note = bar.notes[j];
            notes.push(note);
        }
    }
    return notes;
}

function drawSlurs(ctx, bars) {
    var notes = getListOfNotes(bars);
    // for now, we will assume no nesting of slurs
    var notesStart = [];
    var notesEnd = [];
    var slurCurrent = 0;
    var noteCount = notes.length;
    for (var i = 0; i < noteCount; i++) {
        var note = notes[i];
        if (note.slurNesting > slurCurrent) {
            slurCurrent = note.slurNesting;
            notesStart.push(note);
        }
        else if (note.slurNesting < slurCurrent) {
            slurCurrent = note.slurNesting;
            notesEnd.push(notes[i - 1]);    // we want the previous note
        }
    }
    while (notesStart.length > 0) {
        if (notesStart.length === notesEnd.length) {
            // otherwise something wrong
            drawTie(ctx, notesStart.pop(), notesEnd.pop());
        }
    }
}

function drawStave(ctx, ourTune, line, top, bottom, xPos, lengthScore, firstBar, prev) {
    ctx.beginPath();
    for (var y = top; y <= bottom; y += score.spacing) {
        ctx.moveTo(xPos, y);
        ctx.lineTo(lengthScore + xPos, y);
    }
    ctx.stroke();
    xPos = drawTrebleClef(ctx, xPos, top, bottom);
    xPos = drawKey(ctx, xPos, top, firstBar.flats(), firstBar.sharps(), prev.Flats, prev.Sharps);
//    if (line === 0)
//        xPos = drawMeter(ctx, xPos, top, ourTune.meter);
    return xPos;
}

function getStringName(baseValue, offset)
{
    var names= [ "A", "Bb", "B", "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#" ];
    for (var i = 0; i < names.length; i++)
    {
        if (names[i] === baseValue)
        {
            // i now contains position of base value
            var targetOffset = (offset + i) % names.length;
            return names[targetOffset];
        }
    }
    return 0;
}

function drawTabStave(ctx, top, bottom, xPos, lengthScore, openStringNames) {
    var y, i;
    ctx.beginPath();
    for (y = top; y <= bottom; y += score.spacing) {
        ctx.moveTo(xPos + 30, y);
        ctx.lineTo(lengthScore + xPos, y);
    }
    ctx.stroke();
    y = bottom + score.spacing / 2;
    for (i = 0; i < openStringNames.length; i++) {
        ctx.beginPath();
        ctx.font = score.fontPart;
        ctx.text(openStringNames[i], xPos, y);
        ctx.stroke();
        ctx.fill();
        y -= score.spacing
    }
    return xPos;
}

function calcAdjustment(note, adjustment) {
    //console.log("value: " + note.value);
    if (note.value > 0) {
        var limit = 67;
        var adj = 3;
        if (note.value >= limit) {
            var newAdjustment = settings.score_scale * adj * (note.value - limit - 1);
            if (adjustment < newAdjustment) {
                adjustment = newAdjustment;
                //console.log("value: " + note.value);
                //console.log("adjustment: " + adjustment);
            }
        }
    }
    return adjustment;
}

function calcChordAdjustment(bars) {
    var adjustment = 0;
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var noteCount = bars[i].notes.length;
        for (var j = 0; j < noteCount; j++) {
            var note = bars[i].notes[j];
            if (note.isTuplet) {
                var tuplet = note.getTuplet();
                // TODO: triplet notes too
            }
            else {
                //if (note.hasChord) {
                    adjustment = calcAdjustment(note, adjustment);
                //}
            }
        }
    }
    return adjustment;
}

function bottomAdjustment(note, adjustment) {
    //console.log("value: " + note.value);
    if (note.value > 0) {
        var limit = 46;
        var adj = 3;
        if (note.value < limit) {
            var newAdjustment = settings.score_scale * adj * (limit - note.value - 1);
            if (adjustment < newAdjustment) {
                adjustment = newAdjustment;
                //console.log("value: " + note.value);
                //console.log("adjustment: " + adjustment);
                //console.log("newAdjustment: " + newAdjustment);
            }
        }
    }
    return adjustment;
}

function calcBottomAdjustment(bars) {
    var adjustment = 0;
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var noteCount = bars[i].notes.length;
        for (var j = 0; j < noteCount; j++) {
            var note = bars[i].notes[j];
            if (note.isTuplet) {
                var tuplet = note.getTuplet();
                // TODO: triplet notes too
            }
            else {
                    adjustment = bottomAdjustment(note, adjustment);
            }
        }
    }
    return adjustment;
}

function drawTune(ctx, ourTune, useBeams, openStringValues, openStringNames, smartTabbing) {
    var lines = 5;
    var top = score.topMargin * settings.score_scale;
    top = alignOnHalfPixel(top);
    var padding = score.verticalPadding * settings.score_scale;
    var paddingTab = score.verticalPaddingTab * settings.score_scale;
    var offsetChord = 15 * settings.score_scale;
    var start = score.leftMargin;
    var end = score.width - score.rightMargin;
    var prev = {
        Flats: 0,
        Sharps: 0,
        Meter: "",
        TabMeter: ""
    }
    var repeat = {
        Ending1Start: { x: -1, y: -1 },
        Ending1End: { x: -1, y: -1 },
        Ending2Start: { x: -1, y: -1 },
        Ending2End: { x: -1, y: -1 },
//        Ending1StartPos: -1,
//        Ending1EndPos: -1,
//        Ending2StartPos: -1,
//        Ending2EndPos: -1
    }
    var xPosLastBarEnd = -1
    for (var line = 0; line < ourTune.abclinecollection.length; line++) {
        var countBars = ourTune.abclinecollection[line].bars.length;
        if (countBars)
        {
            var chordAdjustment = calcChordAdjustment(ourTune.abclinecollection[line].bars);
            if (chordAdjustment > 0) {
                top += chordAdjustment;
                top = alignOnHalfPixel(top);
            }
            var topHidden = top;
            ctx.save();
            if (score.hideStaff)
                ctx.globalAlpha = 0.0;
            var firstBar = ourTune.abclinecollection[line].bars[0];
            var meterSaved = prev.Meter;
            var lengthScore = measureScoreLength(ourTune.abclinecollection[line].bars,
                               firstBar.flats(), firstBar.sharps(), prev.Flats, prev.Sharps, prev);
            prev.Meter = meterSaved;
            //console.log("Scorelength: ", lengthScore);
            var bottom = top + score.spacing * (lines - 1);
            ctx.stokeStyle = "black";
            ctx.lineWidth = 1;
            var xPos = start + ctx.lineWidth / 2;
            xPos = drawStave(ctx, ourTune, line, top, bottom, xPos, lengthScore, firstBar, prev);
            prev.Flats = firstBar.flats();
            prev.Sharps = firstBar.sharps();
            //prev.Meter = firstBar.meter;
            //var pitchBar = (end - start + 1 - ctx.lineWidth / 2) / countBars;
            // pitchBar not used, must have been an attempt to even out the bar positions
            // we seem to get crisper lines when xPos is x.5
            for (var i = 0; i < countBars; i++) {
                var bar = ourTune.abclinecollection[line].bars[i];
                if (bar.repeatEndingNumber === 1) {
                    repeat.Ending1Start.x = xPos;
                    bar.repeatEnding1StartX = xPos;
                    repeat.Ending1Start.y = top;
                    bar.repeatEnding1StartY = top;
                }
                else if (bar.repeatEndingNumber === 2)
                {
                    if (repeat.Ending1End.y === top) {
                        repeat.Ending1End.x = xPos;
                        bar.repeatEnding1EndX = xPos;
                        repeat.Ending1End.y = top;
                        bar.repeatEnding1EndY = top;
                    }
                    else {
                        // this fixes issue when repeat 1 is on one line and repeat 2 on next
                        // but doesn't fix if repeat 1 (or 2) carries on to next line
                        repeat.Ending1End.x = xPosLastBarEnd;
                        bar.repeatEnding1EndX = xPosLastBarEnd;
                        repeat.Ending1End.y = top;  // currently not used
                        bar.repeatEnding1EndY = top;  // currently not used
                    }

                    repeat.Ending2Start.x = xPos;
                    bar.repeatEnding2StartX = xPos;
                    repeat.Ending2Start.y = top;
                    bar.repeatEnding2StartY = top;
                }
                if (bar.hasPart)
                    drawPart(bar.part, ctx, xPos, top, bottom);
                else if (bar.hasAnnotation) // for now, only one annotation per bar and in same place as part
                    drawPart(bar.annotation, ctx, xPos, top, bottom);
                bar.xPosStart = xPos;
                xPos = drawStartBar(bar, ctx, top, bottom);
                if (bar.sharps() !== prev.Sharps || bar.flats() !== prev.Flats)
                {
                    xPos = drawKey(ctx, xPos, top, bar.flats(), bar.sharps(), prev.Flats, prev.Sharps);
                    prev.Flats = bar.flats();
                    prev.Sharps = bar.sharps();
                }
                if (bar.meter !== prev.Meter)
                {
                    xPos = drawMeter(ctx, xPos, top, bar.meter);
                    prev.Meter = bar.meter;
                }
                xPos = drawNotesForBar(ctx, bar, xPos, top, bottom, offsetChord + chordAdjustment, useBeams);

                if (repeat.Ending2Start.x !== -1 && bar.isEndOfSecondRepeat())
                {
                    repeat.Ending2End.x = xPos;
                    bar.repeatEnding2EndX = xPos;
                    repeat.Ending2End.y = top;
                    bar.repeatEnding2EndY = top;
                }
                bar.xPosEnd = xPos;
                xPos = drawEndBar(bar, ctx, top, bottom);
                if (repeat.Ending1Start.x !== -1 && repeat.Ending1End.x !== -1)
                {
                    drawRepeatEnding(1, ctx, repeat.Ending1Start, top, repeat.Ending1End);
                    repeat.Ending1Start.x = -1;
                    repeat.Ending1End.x = -1;
                    repeat.Ending1Start.y = -1;
                    repeat.Ending1End.y = -1;
                }
                if (repeat.Ending2Start.x !== -1 && repeat.Ending2End.x !== -1)
                {
                    drawRepeatEnding(2, ctx, repeat.Ending2Start, top, repeat.Ending2End);
                    repeat.Ending2Start.x = -1;
                    repeat.Ending2End.x = -1;
                    repeat.Ending2Start.y = -1;
                    repeat.Ending2End.y = -1;
                }
                xPosLastBarEnd = xPos;  // save it for repeat ending
            }
            // now draw any ties
            drawTies(ctx, ourTune.abclinecollection[line].bars, false);
            // TODO: get slurs corrected
            //drawSlurs(ctx, ourTune.abclinecollection[line].bars);
            ctx.restore();
            if (openStringValues.length > 0)
                top += score.spacing * lines + paddingTab;
            else
                top += score.spacing * lines + padding;
            var bottomAdj = calcBottomAdjustment(ourTune.abclinecollection[line].bars);
            //if (bottomAdj !== 0)
            //    console.log("bottomAdj:" + bottomAdj + "(" + line + ")");
            top += bottomAdj;
            top = alignOnHalfPixel(top);
            if (score.hideStaff)
                top = topHidden;
            if (openStringValues.length > 0) {

                for (i = 0; i < countBars; i++) {
                    bar = ourTune.abclinecollection[line].bars[i];
                    calcTabNotesForBar(bar, openStringValues);
                }
                if (smartTabbing)
                    ourTune.optimiseFingering(openStringValues.length);

                xPos = start + ctx.lineWidth / 2;
                bottom = top + score.spacing * (openStringValues.length - 1);
                ctx.lineWidth = 1;
                drawTabStave(ctx, top, bottom, xPos, lengthScore, openStringNames);
                var tupletNumberDrawn = false;
                for (i = 0; i < countBars; i++) {
                    bar = ourTune.abclinecollection[line].bars[i];
                    if (bar.repeatEnding1StartX !== -1) {
                        repeat.Ending1Start.x = bar.repeatEnding1StartX;
                        repeat.Ending1Start.y = bar.repeatEnding1StartY;
                    }
                    if (bar.repeatEnding1EndX !== -1) {
                        repeat.Ending1End.x = bar.repeatEnding1EndX;
                        repeat.Ending1End.y = bar.repeatEnding1EndY;
                    }
                    if (bar.repeatEnding2StartX !== -1) {
                        repeat.Ending2Start.x = bar.repeatEnding2StartX;
                        repeat.Ending2Start.y = bar.repeatEnding2StartY;
                    }
                    if (bar.repeatEnding2X !== -1) {
                        repeat.Ending2End.x = bar.repeatEnding2EndX;
                        repeat.Ending2End.y = bar.repeatEnding2EndY;
                    }
                    var offset = score.spacing / 2 - 2;
                    xPos = drawStartBar(bar, ctx, top, bottom);
                    if (bar.meter !== prev.TabMeter)
                    {
                        xPos = drawMeter(ctx, xPos, top, bar.meter);
                        prev.TabMeter = bar.meter;
                    }
                    if (drawTabNotesForBar(ctx, bar, top + offset, bottom + offset))
                        tupletNumberDrawn = true;
                    drawEndBar(bar, ctx, top, bottom);
                    if (repeat.Ending1Start.x !== -1 && repeat.Ending1End.x !== -1)
                    {
                        if (score.hideStaff)
                            drawRepeatEnding(1, ctx, repeat.Ending1Start, top, repeat.Ending1End);
                        repeat.Ending1Start.x = -1;
                        repeat.Ending1End.x = -1;
                        repeat.Ending1Start.y = -1;
                        repeat.Ending1End.y = -1;
                    }
                    if (repeat.Ending2Start.x !== -1 && repeat.Ending2End.x !== -1)
                    {
                        if (score.hideStaff)
                            drawRepeatEnding(2, ctx, repeat.Ending2Start, top, repeat.Ending2End);
                        repeat.Ending2Start.x = -1;
                        repeat.Ending2End.x = -1;
                        repeat.Ending2Start.y = -1;
                        repeat.Ending2End.y = -1;
                    }
                }
                drawTies(ctx, ourTune.abclinecollection[line].bars, true);
                top += score.spacing * openStringValues.length + padding;
                if (tupletNumberDrawn)
                    top += score.spacing;
                top = alignOnHalfPixel(top);
            }
        }
    }
}

function alignOnHalfPixel(yValue) {
    return Math.floor(yValue) + 0.5
}

function calcScoreWidth(tune) {
    var lengthMax = 0;  // start small
    if (tune !== null) {
        var prev = {
            Flats: 0,
            Sharps: 0,
            Meter: ""
        }
        for (var line = 0; line < tune.abclinecollection.length; line++) {
            var prevFlats = 0;
            var prevSharps = 0;
            if (tune.abclinecollection[line].bars.length > 0) {
                var firstBar = tune.abclinecollection[line].bars[0];
                var lengthScore = measureScoreLength(tune.abclinecollection[line].bars,
                                   firstBar.flats(), firstBar.sharps(), prevFlats, prevSharps, prev);
                if (lengthScore > lengthMax)
                    lengthMax = lengthScore
                prevFlats = firstBar.flats();
                prevSharps = firstBar.sharps();
            }
        }
    }
    // add 10 for a little bit of
//    console.log("calcScoreWidth lengthMax: " + lengthMax);
//    console.log("flick width: " + flick.width);
    var result = score.iosScale * (lengthMax + (score.leftMargin + score.rightMargin) * settings.score_scale);
    //console.log("score width: " + result);
    return result;
}

function containsTuplet(bars) {
    var countBars = bars.length;
    for (var i = 0; i < countBars; i++) {
        var noteCount = bars[i].notes.length;
        for (var j = 0; j < noteCount; j++) {
            var note = bars[i].notes[j];
            if (note.isTuplet) {
                return true;
            }
        }
    }
    return false;
}

function calcScoreHeight(tune) {
    var line;
    var padding = score.verticalPadding * settings.score_scale;
    var paddingTab = score.verticalPaddingTab * settings.score_scale;
    var lines = 5;
    var top = score.topMargin * settings.score_scale;
    var hideStaff = false;
    var legLength = 0;
    if (tune !== null) {
        if (score.tuningVector.length > 0) {
            if (tune.hideStaffWhenTab)
                hideStaff = true;
        }
        if (tune.showLegs)
            legLength = score.legLength;
        for (line = 0; line < tune.abclinecollection.length; line++) {
            if (tune.abclinecollection[line].bars.length > 0) {
                var yAdjustment = calcChordAdjustment(tune.abclinecollection[line].bars);
                if (yAdjustment === 0) {
                    // maybe we are going off the bottom
                    yAdjustment = calcBottomAdjustment(tune.abclinecollection[line].bars);
                    //if (yAdjustment !== 0)
                    //    console.log("yAdjustment:" + yAdjustment + "(" + line + ")");
                }

                if (hideStaff)
                    top += padding;
                else
                    top += score.spacing * lines + padding + yAdjustment;
                if (score.tuningVector.length > 0) {
                    // a tab is showing
                    top += score.spacing * score.tuningVector.length + 1;
                    if (!hideStaff)
                        top += paddingTab;
                    // if leg length is non-zero, then legs are showing
                    top += legLength;
                    // if there is a tuplet, then allow for number
                    if (containsTuplet(tune.abclinecollection[line].bars)) {
                        top += score.spacing;
                    }
                }
            }
        }
    }
    var result = score.iosScale * top;
    //console.log("score height: " + result);
    return result;
}
